.comment Standard DECUS C Documentation layout
.comment
.comment Position the printer to start on the 4th line for 66 line
.comment (11 inch) paper.  The 60 lines printed should be vertically
.comment centered on the page, with 3 line vertical margins top and bottom.
.comment
.comment The title page is laid out so that the title will show through
.comment a "window" cover such as is used on Digital documentation.
.comment
.comment Note: there is a bug in the page number algorithm of RSTS/E
.comment runoff.  Thus, on RSTS/E, GETRNO.C must mangle the page
.comment heading to fix page number alignment.
.comment
.comment The following statement will give a (non-fatal) error on RNO.TSK
.comment It is needed because somebody was clever in designing "Standard
.comment RUNOFF".
.NO AUTOSUBTITLE .STYLE HEADERS 3,0,0
.comment
.nhd.uc.ps 58,80.lm 8.rm 72
#
.s 26
.c ;DECUS C LANGUAGE SYSTEM
.s 2
.comment else
.c ;Runtime Support Library
.c ;Reference Manual
.comment mundane
.s 2
.c ;by
.s
.c ;Martin Minow
.s 2
.c ;Edited by
.s
.c ;Robert B. Denny
.s 3
This document describes the RSX/VMS/RSTS/RT11 DECUS C language system
runtime support library.
.comment else
.s 2
.comment mundane
.s 2
.c ;DECUS Structured Languages SIG
.s
.c ;Version of 15-Oct-81
.nhy
.p 2, 1, 3
.note
This software is made available without any support whatsoever.
The person responsible for an implementation of this system
should expect to have to understand and modify the source code if
any problems are encountered in implementing or maintaining the compiler or
its run-time library.  The DECUS 'Structured Languages Special Interest
Group' is the primary focus for communication among users of this software.
.en
.s
UNIX is a trademark of Bell Telephone Laboratories.  RSX, RSTS/E, RT11
and VMS are trademarks of Digital Equipment Corporation.
.s
.hd
.hd mixed
.comment The following gives a bug on RNO.TSK
.head mixed
.comment
.ch INTRODUCTION
.ps 58,80.lm 8.rm 72
.hd mixed
.comment else
.t ########C Runtime Support Library
.st ########Reference Manual
.comment mundane
The C support library contains three major groups of functions:
.lm +8
.s.i -3;o##Functions called by the compiler to perform certain mathematical
operations (such as floating-point multiply or function entrance) that
are not performed in-line.
.s.i -3;o##Functions that may be called by C programs to perform operations,
such as copying a string, that are more efficiently performed by optimized
assembly language subroutines.
.s.i -3;o##An implementation of (most of) the Unix Standard I/O library.
.s.lm -8
.comment mundane
.ch THE STANDARD LIBRARY
.ps 58,80.lm 8.rm 72
.hd mixed
.comment else
.t ########C Runtime Support Library
.st ########Reference Manual
.comment mundane
.note WARNING
This version of the library is somewhat incompatible with previous
versions of the Decus C compiler.
This incompatibility allows greater compatibility
with Unix Version 7, and the capability of generating global references
and definitions containing the radix-50 '$' and '.' characters.
.en
.s
All C programs should be linked together with the run-time support
library.  On systems that support logical device descriptors, this
will generally be stored in the C: logical device/account.  On RSX-11M,
the library will be found in LB:[1,1].  To link a C program, proceed
as follows:
.lm +16.nf
.s.i -8;RSX	TKB
TKB> task/CP,map=objects,LB:C/LB
TKB> /
ENTER OPTIONS:
TKB> STACK=2000
TKB> //
.s.i -8;RT11	R LINK
*save,map=objects,C:SUPORT,C:CLIB/B:2000
*_^C
.f.s.lm -16
Note that, on RSX-11M, the task should be built checkpointable.  This
is automatic on RSTS/E and VMS emulated modes.  On native RSX-11M,
the task-builder may be rebuilt with '/CP' as the default.  If the
'/CP' switch is not given, the task may abort (no memory) when started.
Stack extension is needed if subroutines locally allocate large amounts of
data.
.s
On RT11, note that the '/B' (bottom) switch must be given to increase
the size of the run-time stack.  This is needed as local variables are
allocated on the stack.  If the stack size is not increased, the program
may abort without any error message.
.s
The standard I/O interface header file is in C:STDIO.H
(LB:[1,1]STDIO.H on native RSX-11M implementations)
and should be included in
all C programs as follows:
.s
		_#include <stdio.h>
.hl 1 ^&Introduction to Unix-style Stream I/O\&
.s
This section presents an overview of the 'standard' I/O interface
for C programs.  These routines have proven easy to use and,
while they do not provide the full functionality of RSX or RMS, they give
the programer essentially all that is needed for everyday use.
The discussion includes the following:
.s
.nf
	- Opening and closing files
	- Reading data from files
	- Writing data to files
.f
.s
Note that this file system is limited:  files are sequential
with only a limited amount of random-access capability.
Also, little of the power of RMS/FCS is available.
On the other hand, the limitations make C programs easily transportable
to other operating systems.
.s
When a C program is started, three files are predefined and opened:
.s
	stdin####The 'standard' input file
.br
	stdout###The 'standard' output file
.br
	stderr###The 'standard' error file
.s
Stderr is always assigned to the command terminal.  Stdin and stdout
are normally assigned to the command terminal, but may be reassigned
or closed when the program starts.
.hl 1 ^&Opening and Closing Files\&
The fopen and fclose functions control access to files.
They are normally used as follows:
.s
.nf
	_#include _<stdio.h_>	/* Define standard I/O	*/
	FILE	*ioptr;		/* Pointer to file area	*/
	...
	if ((ioptr = fopen("filename", "mode")) == NULL)
		error("Can't open file");
.f
.s
All information about an open file is stored in a buffer
whose address is returned by fopen().
The buffer is dynamically allocated when the file is opened and
deallocated when the file is closed.
Its format is described under the heading IOV.
The mode string contains flags defining how the file is to be accessed:
.s
.nf
	r	Read only
	w	Write new file
	a	Append to existing file (or write new)
		(Doesn't work on RT11 modes).
.s
It also defines some record-control information:
.s
	n	No newlines mode ('binary' files)
	u	RSX:  The user program allocates record buffers
		RT11: Console output is done using .ttyout
.f
.s
If mode 'n' is specified, the RSX I/O routines do not interpret the
end of a RMS logical record as defining the end of a human-readable
line.  This is necessary to process 'binary' information.
On RT11, the I/O routines do not remove carriage-returns from
input files, nor do they insert carriage-returns
when newlines are output.
.s
Mode 'u' is treated differently on RSX and RT11. If specified on
RSX the user maintains the file's record buffer.  The program may
only call the fget() and fput() routines. Each such
call will be translated directly to an RMS/FCS GET$ or PUT$ call.
(In RT11 mode, fput() may
be used to write binary records which are subsequently read by fget().
The file should be opened with 'n' mode.)
.s
If mode 'u' is specified on RT11 mode when opening the console terminal,
single character output will be performed (using the RT11 monitor routine
_.ttyout).  If not specified, output will be one line at a time (using
the monitor routine _.print).  The library initialization process opens
stderr in mode 'u' and stdout in normal mode.  This means that mixing
I/O to both units will result in synchronization problems.  To obtain
single-character output on stdout, the program need simply execute:
.s
	if (freopen("tt:", "wu", stdout) == NULL) error("");
.s 2
The program calls fclose(ioptr) to terminate processing on a file.
This closes the file and reclaims system-controlled buffer space.
fmkdl(ioptr) may be called to close and delete an open file.
.hl 1 ^&Reading Data from a File\&
The simplest way to read a file is to call the getchar() or getc() routines
which read the next character from a file.  For example, the following
program counts the number of characters in a file:
.s
.nf
	_#include _<stdio.h_>
	main()
	{
		register int	ccount;
		register int	lcount;
		register int	c;
		FILE		*ioptr;

		if ((ioptr = fopen("foo.bar", "r")) == NULL)
			error("Cannot open file");
		ccount = 0;
		lcount = 0;
		while ((c = getc(ioptr)) != EOF) {
			ccount++;
			if (c == '_\n') lcount++;
		}
		printf("_%d characters, _%d lines._\n",
				ccount, lcount);
	}
.f
.s
Other input routines include:
.s
.nf
	gets	Read a line from the standard input
	fgets	Read a line from a file
	fgetss	Read a line from a file, remove terminator
	ungetc	Push one character back on a file
	fseek	Position the file to a specific record
.f
.s
These routines are used together with the ferr() and feof() functions
which allow testing for error and end of file conditions, respectively.
The package assumes that all error conditions are lethal and force end
of file.
.hl 1 ^&Writing Data to a File\&
There are several routines for data output which are directly analagous
to the data input routines:
.s
.nf
	putchar	Write a character to standard output
	putc	Write a character to a specified file
	puts	Write a string to the standard outpt
	fputs	Write a string to a specified file
	fputss	Write a record to a specified file
	ftell	Return record location (for fseek)
.f
.s
In addition, the printf() or fprintf() routine is used to format
and print data (as was shown in the previous example).  Printf()
is flexible, powerful, and easy to use; and is perhaps the
single most important routine in the file system.
.hl 1 ^&Interaction with the Operating System\&
The support library attempts to provide a consistant operating
system interface on several implementations of
quite different operating systems.  The possiblities include:
.lm +8
.s.i -3;o##Native RT11
.s.i -3;o##Native RSX-11M or IAS
.s.i -3;o##RT11####emulated on RSTS/E
.s.i -3;o##RSX-11M emulated on RSTS/E
.s.i -3;o##RSX-11M emulated on VAX/VMS
.s.lm -8
This section mentions the inconsistencies and random bugs that are
more or less intentionally present.
.hl 2 ^&Logical Unit Numbers\&
.s
RT11, RSTS/E, and RSX-11M use small integers to number all I/O
channels.  Unfortunately, the numbering is not consistent and
channel usage differs among the various systems.  This has resulted
in several quirks:
.lm +8
.s.i -3;o##On RT11, console terminal interaction uses
the _.ttyout and _.print monitor functions.  While no device is opened
by the fopen() function, a channel number is reserved.
.s.i -3;o##On RSX-11M, a channel must be allocated to the console terminal.
When a C program starts, the 'command terminal' is opened on logical unit
1 and assigned to stderr.  This is not done using the fopen() routine
(although the stderr IOV structure is defined as if fopen() were called).
Also, fclose() cannot close stderr.
.s
In addition to the standard I/O routines, there are two routines,
msg() and regdmp(), that direct output to logical unit 1.  These routines
were used to debug the library, and are otherwise useful for disasters.
On RT11, msg() and regdmp() use the _.print monitor function.
.s.i -3;o##On both systems, the first true files are stdin and stdout.
These are opened on logical units 0 and 1 (RT11) or 2 and 3 (RSX).
Code in fclose() double-checks that the logical unit number of the
file being closed corresponds to the proper slot in the logical unit
table.
.s.i -3;o##Since the standard I/O routines know little about the
operating system, they do not deal with certain special features.
.s
For example, on RT11, the 'user service routine' (USR) is used to open
files.  The file open routine does not attempt to locate IOV areas
so that the USR does not swap over them.  This can be a problem for
very large programs.
.s
On RSX, logical unit numbers are assigned dynamically.  This means
that 'LUN assignment' cannot be reliably performed by task-build
control files (or task initiation).
.lm -8
.hl 2 ^&Wild-Card Files\&
.s
The fwild()
and fnext() routines can be used to process 'wild-card' files.
.s
On RSX-11M and VMS/RSX emulation, the file name, extension, and/or
file version may be expressed as '*' to indicate wild-card support.
Due to restrictions in the RSX-11 Files-11 structure (ODS1), the
' ', ';0' and ';-1' version numbers will NOT function properly
on native RSX-11. This means that '*.MAC' will not work, for
instance. The algorithm in fwild() depends on the directory being
sorted, as in the ODS2 file structure used on VAX/VMS. If
you sort the directory to be used in an fwild() operation (use the
SRD program available from DECUS with the /WB switch) in order of
decreasing file version numbers, then the " ", ";0", and ";-1"
versions will work.
.s
Wild-card devices, units, or UIC codes are not supported.
Note that, on RSX11 emulated on VMS, the ';-1' (earliest version)
requires special processing by the user program, as noted in the
description of fwild().
.s
On RSX-11M emulated on RSTS/E, the file name and/or extension may
contain '*' or '?' wild-cards.  Wild-card devices, units, or UIC (PPN)
codes are not supported.
.s
On RT-11, the file name and/or extension may contain '*', '%' or '?'
wild cards. The '?' wild-card is synonymous with '%'. Wild-card devices,
or units are not supported. Since wild-card support is not an RT-11
system service, the file specification
parsing and directory operations are handled
by the C library fwild() and fnext() functions. The fwild/fnext module
requires about 480 words of storage. Also, a file opened by wild-card
uses a 1024 byte buffer, twice the normal size, which is shared between
data and directory operations.
.hl 2 ^&Memory allocation\&
.s
Memory is allocated using the malloc() function.  It works
correctly on all operating systems, expanding memory on RSX and RT11/RSTS.
On 'native' RT11, all available memory is allocated by invoking
the monitor _.settop function with a '-2' argument.  Neither library
supports extended memory (PLAS) directives.
.s
The sbreak() routine may be used for permanent allocation of memory.
Once memory has been allocated using sbreak(), it cannot be freed.
.hl 2 ^&Global symbols\&
Global symbols defined or referenced by C programs may cause unexpected
parts of the operating-system support library to be loaded, with
undesirable consequences.  There are several aspects to this problem:
.lm +8
.s.i-3;o##A symbol refered to by a C program which is not resolved
by the object modules or libraries may cause some part of the system
library to be loaded.  For example, if a C program refers to the
sqrt() function (which is not present in the standard library),
the linker may attempt to load the function from the Fortran library.
Since the Fortran sqrt() function contains a reference to the Fortran
run-time error message routine, the load map may contain many strange
global symbols.
.s.i -3;o##Another problem occurs when a C program defines a global
symbol that duplicates a global in the system library.  This may
prevent loading some necessary part of the system library, causing
the link to fail with unresolved global symbols.  For example, on
RSX-11M emulated on RSTS/E V7.0, the following will fail:
.s.nf
	int eof;	/* Global	*/
	main () {
	}
.s.f
.lm -8


.comment mundane
.ch LIBRARY FUNCTION DESCRIPTIONS
.ps 58,80.lm 8.rm 72
.hd mixed
.comment else
.t ########C Runtime Support Library
.st ########Reference Manual
.comment mundane
This chapter contains descriptions of each of the C-callable runtime library
functions.
.comment mundane
.s
In most cases the I/O functions match those of UNIX V7 closely, with
differences normally called out. For more information, consult
"The C Programming Language" by Brian Kernighan and Dennis Ritchie
(Englewood Cliffs, NJ: Prentice-Hall, ISBN#0-13-110163-3).
.s
.comment mundane
.lm 8.rm 72.nhy

.st ########$$ctim##Convert $$rtim Buffer to Ascii
.pg
.hl 1 ^&Convert $$rtim Buffer to Ascii\&
.s 2
.c ;**********
.c ;* $$ctim *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
$$ctim(buffer);
struct TIME {
	int	year;	/* G.TIYR  year - 1900	*/
	int	month;	/* G.TIMO  Jan. = 1	*/
	int	day;	/* G.TIDA		*/
	int	hour;	/* G.TIHR  00 .. 23	*/
	int	minute;	/* G.TIMI  00 .. 59	*/
	int	second;	/* G.TISC  00 .. 59	*/
	int	tick;	/* G.TICT  00 .. tsec-1	*/
	int	tsec;	/* G.TICP  tick/second	*/
} buffer;
.s
dayofweek(buffer);
struct TIME buffer;
.s
.i -8;Description
.s
.fill
$$ctim() converts the time information in buffer such as returned by
$$rtim() to a character string in the format:
.s
.nf
	Sun Sep 16 01:03:52 1973_\0_\0
.s
.fill
All the fields have constant width.  Note that there are two
trailing <null>'s for the benefit of programs converted from
Unix (where the year is followed by a <newline>).
.s
$$ctim(0) returns the current time of day, calling $$rtim() internally.
.s
The format of the returned information is identical to
the Unix function of the same name except that the Unix function
has a trailing newline which is omitted here.
.s
dayofweek() returns the day of the week (Sunday = 0) for the
date information in the argument buffer.
.s
.i -8;Bugs
.s
There is no range checking on the information passed.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$init##One-time initialization code
.pg
.hl 1 ^&One-time initialization code\&
.s 2
.c ;**********
.c ;* $$init *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;Description
.s
.fill
When a C program is started, a command line is parsed
to form the argv[] array and the standard input, output,
and error files are opened.  Note the following:
.s
On RSX11/M (even emulated), argv[0] will be set to the
task name.  On RT11 modes, argv[0] will be set to a dummy
value.
.s
If no command line is passed to the program, it will prompt
the task name (or "Argv") and read a line from the command
terminal.  To disable this, the user program may define
a global flag as follows:
.s
.nf
	$$narg = 1;
	main (argv, argc) {
	...
.s
.fill
If $$narg is initialized non-zero and no command line is passed
to the program, or if the initialization sequence fails to
read an argument, main() will be called with one argument.
.s
The command line prompt may be changed from the task name (or
"Argv") by defining a global string as follows:
.s
.nf
	char *$$prmt = "Command line prompt"
	main(argv, argc) {
	...
.s
.fill
On Vax VMS, the program may be installed as a "foreign command"
by a command such as:
.s
.nf
	$ command :== $disk:[dir]filename
.s
.fill
and executed by typing:
.s
.nf
	$ command arg1 arg2
.s
.i -8;Diagnostics
.s
.fill
Can't parse command line
.s
?C-Standard input, [filename]: error text
.s
?C-Standard output, [filename]: error text
.s
?C-No memory.
.s
The "can't parse" message is given if the command line
format is incorrect (because of unbalanced quotation marks, for
example).
.s
The "standard input" or "standard output" messages are a
user error if input or output are redirected and the associated
files cannot be opened.  The text should be self-explanatory.
.s
The "no memory" message suggests a severe
case of program immensity.
.s
All errors are fatal.
.s
.i -8;Bugs
.s
On RSTS/RSX, command lines are limited to 80 bytes, although
128 byte commands are feasable.  On VMS V3.0 and V3.1, the command
name includes the expanded form of the command name.  This
means that long command name translations coupled with long
argument strings may cause the total argument string to exceed
80 bytes.  Such arguments will be truncated by the operating
system without warning.  The work-around is to make use
of logical names:
.s
.nf
	$ assign $disk[directory.subdirectory] bin
	$ program :== $bin:prog
.s
.fill
The expanded form of the "program" command is just "bin:prog".
.s
On VMS V3.0 and V3.1, the distributed RSX file service (FCS) refuses
to open "stream Ascii" files.  This module contains a
dynamically-installed patch to the open routine.  An error
message will be printed if the patch cannot be installed
correctly.  The VMS3.0 compile-time switch enables this
patch.  There is also code in fopen.mac that is affected
by this patch.
.s
It might be reasonable to make error messages non-fatal
so $$init could be called by a user program.
.s
On RSX, the program aborts (by executing a BPT instruction)
if the program fails to open stderr or obtain partition parameters.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$main##Start of C programs
.pg
.hl 1 ^&Start of C programs\&
.s 2
.c ;**********
.c ;* $$main *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
$$main::		/* Start of C programs	*/
.s
extern int   $$opsy;	/* Operating system	*/
extern int   $$rsts;	/* Non-zero if RSTS/E	*/
extern int   $$vms;	/* Non-zero if VMS	*/
extern int   $$pos;	/* Non-zero if P/OS	*/
extern int   $$uic;	/* RSX: Default uic	*/
extern int   $$stnd;	/* Stack end point	*/
extern int   $$scca;	/* RT11: .scca control	*/
extern char  $$task[];	/* RSX: task name	*/
extern int   $$tick;	/* Clock interrupt rate	*/
.s
.i -8;Description
.s
.fill
This module contains the run-time startoff for C programs.
It intializes the C environment and calls the main() subroutine.
On return from the main program, exit() is called to close
all files and return to the operating system.
.s
The following globals are defined in this module:
.s
.nf
	$$opsy	Operating  system  unique  value.   This
		value  is  7 on RT11, and set as per the
		GTSK$ directive	on RSX-based systems:
			RSX11-D		0
			RSX11-M		1
			RSX11-S		2
			IAS		3
			RSTS/E V7	4
			VMS		5
			RT11		7
			P/OS		9
.s
	$$rsts	This value is non-zero if the program is
		running  on  RSTS/E.   It is zero if the
		program is running native RSX,  RT11  or
		VMS compatibility mode.
.s
	$$vms	This value is non-zero if the program is
		running  (in  compatibility mode) on Vax
		VMS.  It  is  zero  if  the  program  is
		running native RSX, RT11 or on RSTS/E.
.s
	$$pos	This value is non-zero if the program is
		running on the Professional P/OS.  It is
		zero if the program  is  running  native
		RSX, RT11, or on RSTS/E or VMS.
.s
	$$stnd	The (top) end of the stack on entrance.
.s
	$$scca	On RT11, the .scca control  word.   This
		is  needed  to  allow RSTS/E programs to
		trap end of file.
.s
	$$task	On RSX, the task name in Ascii.
.s
	$$tick	The clock interrupt rate.  This  is  set
		at  initialization  on  RT11  and on the
		first call to time() or ftime() on RSX.
.s
	$$uic	On RSX, the default UIC word from GTSK$.
.s
.i -8;Diagnostics
.s
.fill
On RSX, the task aborts with a BPT if the standard error file
did not open.  Otherwise, explanatory error messages are printed.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$narg##Define default for $$narg
.pg
.hl 1 ^&Define default for $$narg\&
.s 2
.c ;**********
.c ;* $$narg *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
int	$$narg	= <value>;
_...
main() {
	...
}
.s
.i -8;Description
.s
.fill
If no command line is provided when the program
starts, the program will print a prompt on the command
terminal, read one line and build the argv[] vector.
If this is undesirable, include a global definition of
$$narg in your C program as follows:
.s
.nf
	int	$$narg	=	1;
.s
.fill
This will supress the command terminal read operation.
The argv[] vector will be set to a dummy value.
.s
If the user program does not define $$narg, it will be defined
so as to enable the prompt.
.s
Note that $$narg must be initilized at compilation time.
It is tested by the initialization code before the main()
program is called.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$prmt##Null prompt pointer
.pg
.hl 1 ^&Null prompt pointer\&
.s 2
.c ;**********
.c ;* $$prmt *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
char *$$prmt = "Prompt string";
.s
.i -8;Description
.s
.fill
This pointer is checked in $$init and if the user has defined
$$prmt then it will be used as the task prompt instead of
using the task name on RSX or "Argv:" on RT-11.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$rtim##Date and time conversion
.pg
.hl 1 ^&Date and time conversion\&
.s 2
.c ;**********
.c ;* $$rtim *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
$$rtim(buffer);
struct TIME {
	int	year;	/* G.TIYR  year - 1900	*/
	int	month;	/* G.TIMO  Jan. = 1	*/
	int	day;	/* G.TIDA		*/
	int	hour;	/* G.TIHR  00 .. 23	*/
	int	minute;	/* G.TIMI  00 .. 59	*/
	int	second;	/* G.TISC  00 .. 59	*/
	int	tick;	/* G.TICT  00 .. tsec-1	*/
	int	tsec;	/* G.TICP  tick/second	*/
} buffer;
.s
.i -8;Description
.s
.fill
Stores the current time of day in the buffer.  The format
is compatible with the RSX11-M GTIM$ executive service.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$utim##Convert $$rtim buffer to Unix time
.pg
.hl 1 ^&Convert $$rtim buffer to Unix time\&
.s 2
.c ;**********
.c ;* $$utim *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
long
$$utim(buffer);
struct TIME {
	int	year;	/* G.TIYR  year - 1900	*/
	int	month;	/* G.TIMO  Jan. = 1	*/
	int	day;	/* G.TIDA		*/
	int	hour;	/* G.TIHR  00 .. 23	*/
	int	minute;	/* G.TIMI  00 .. 59	*/
	int	second;	/* G.TISC  00 .. 59	*/
	int	tick;	/* G.TICT  00 .. tsec-1	*/
	int	tsec;	/* G.TICP  tick/second	*/
} buffer;
.s
.i -8;Description
.s
.fill
The user calls $$rtim() to initialize a buffer.
Then, $$utim will convert buffer contents to the
number of seconds since Jan 1, 1970.
.s
Note: $$utim() does not compensate for local timezone
or daylight savings time.
.s
.i -8;Bugs
.s
This routine works only in the range:
.s
.nf
	1970 <= year < 2100
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########abort###Abort program with a BPT trap
.pg
.hl 1 ^&Abort program with a BPT trap\&
.s 2
.c ;*********
.c ;* abort *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
abort()
.s
.i -8;Description
.s
.fill
After closing all files, the program exits
by executing a BPT trap.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########abs#####Integer absolute value
.pg
.hl 1 ^&Integer absolute value\&
.s 2
.c ;*******
.c ;* abs *
.c ;*******
.s 2
.lm +8.nf
.i -8;Usage
.s
abs(val)
int		val;
.s
.i -8;Description
.s
.fill
Return absolute value of the integer argument.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########alloc###Allocate memory
.pg
.hl 1 ^&Allocate memory\&
.s 2
.c ;*********
.c ;* alloc *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
alloc(n);		/*   -1 if no space	*/
.s
.i -8;Description
.s
.fill
Allocate the requested number of bytes, returning a pointer
to the first.  The program image will be expanded as needed.
.s
alloc() returns -1 if the requested space could not be allocated.
.s
free() is used to return the buffer to free space.
.s
See also the description of malloc(), realloc() and sbrk().
.s
.i -8;Diagnostics
.s
.i -8;Bugs
.s
alloc() is obsolete.  Use malloc() for new programs.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ascr50##Ascii to Radix-50 conversion
.pg
.hl 1 ^&Ascii to Radix-50 conversion\&
.s 2
.c ;**********
.c ;* ascr50 *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
ascr50(count, input, output)
int		count;
int		*output;
char		*input;
.s
.i -8;Description
.s
.fill
Convert 'count' characters from ascii to Radix 50.  If there aren't a
multiple of 3 characters, blanks are padded on the end.  The ascii
string is moved to the output when finished.  The input and output
areas may be the same since three input characters are read for
every two bytes of output.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########atod####Convert Ascii to double floating
.pg
.hl 1 ^&Convert Ascii to double floating\&
.s 2
.c ;********
.c ;* atod *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
double
atod(buffer)
char		*buffer;
.s
.i -8;Description
.s
.fill
Converts a double-precision floating point number written in
scientific notation from ASCII to a "double" number.  The BNF
for numbers which can decoded by this routine follows:
.s
.nf
    <number> := <sign> <real number>
    <real number> :=
	<decimal number>
	| <decimal number> <exponent>
	| <exponent>
    <decimal number> :=
	<integer>
	| <integer> .
	| . <integer>
	| <integer> . <integer>
    <integer> := <digit> | <integer> <digit>
    <exponent> :=
	{ E | e | D | d } <sign> <integer>
    <digit> := 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
    <sign> := + | - | <empty>
.s
.fill
The conversion stops on the first non-legal character.  (Note:
if this routine is declared type char *, it will return a pointer
to the first non-legal character.)
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########atof####Convert Ascii to double floating
.pg
.hl 1 ^&Convert Ascii to double floating\&
.s 2
.c ;********
.c ;* atof *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
double
atof(buffer)
char		*buffer;
.s
.i -8;Description
.s
.fill
Converts a double-precision floating point number written in
scientific notation from ASCII to a "double" number.  The BNF
for numbers which can decoded by this routine follows:
.s
.nf
    <number> := <sign> <real number>
    <real number> :=
	<decimal number>
	| <decimal number> <exponent>
	| <exponent>
    <decimal number> :=
	<integer>
	| <integer> .
	| . <integer>
	| <integer> . <integer>
    <integer> := <digit> | <integer> <digit>
    <exponent> :=
	{ E | e | D | d } <sign> <integer>
    <digit> := 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
    <sign> := + | - | <empty>
.s
.fill
The conversion stops on the first non-legal character.  (Note:
if this routine is declared type char *, it will return a pointer
to the first non-legal character.)
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########atoi####Convert Ascii to integer
.pg
.hl 1 ^&Convert Ascii to integer\&
.s 2
.c ;********
.c ;* atoi *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
atoi(p)
char		*p;
.s
.i -8;Description
.s
.fill
Convert string to integer.  No data checking is performed.
The conversion stops at the first non-integer.  Leading
spaces, tabs, plus-signs and/or newlines are ignored.
The integer may be preceeded by a minus sign.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########atol####Convert Ascii to long
.pg
.hl 1 ^&Convert Ascii to long\&
.s 2
.c ;********
.c ;* atol *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
long
atol(p)
char		*p;
.s
.i -8;Description
.s
.fill
Convert string to integer.  No data checking is performed.
The conversion stops at the first non-integer.  Leading
spaces, tabs, plus-signs and/or newlines are ignored.
The integer may be preceeded by a minus sign.
.s
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########brk#####Change memory allocation
.pg
.hl 1 ^&Change memory allocation\&
.s 2
.c ;*******
.c ;* brk *
.c ;*******
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
brk(addr);
int		*addr;
.s
.i -8;Description
.s
.fill
brk() sets the run-time library's "top of memory" pointer
to addr.  This value should have been returned by a previous
call to sbrk().  Intervening calls to malloc() or file open
routines may cause memory to be corrupted by calling brk().
.s
.i -8;Diagnostics
.s
Returns 0 if the brk() argument is reasonable.  Returns -1
if it is greater than the current top of memory.  (Use sbrk()
to increase memory allocation.)
.s
.i -8;Bugs
.s
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########call####Call (Fortran) subroutine from C
.pg
.hl 1 ^&Call (Fortran) subroutine from C\&
.s 2
.c ;********
.c ;* call *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
return__value = call(routine, count, _&arg1, ... _&argN);
extern int routine();	/* PDP-11 function name	*/
int	   count;	/* Number of arguments	*/
.s
.i -8;Description
.s
.fill
C Library user-callable interface between C programs and
subroutines which follow the standard PDP11 calling sequence.
Note that call() saves and restores registers r2-r5.
.s
The arguments are as follows:
.s
.nf
	routine		The name of the PDP11 routine to
			be called.
.s
	count		The number of arguments  in  the
			calling sequence.
.s
	arg1		The first argument.
.s
.fill
For example:
.s
.nf
	extern int	sort();
	int		data__to__sort[NDATA];
	int		ndata = NDATA;
.s
	call(sort, 2, _&data__to__sort, _&ndata);
.s
.fill
Note that, if the called function returns some other datatype
such as a long, the routine and call() must be correctly declared:
.s
.nf
	long		call();
	long		myfun();
	long		larg;
	long		result;
.s
	result = call(myfun, 1, _&larg);
.s
.fill
If call() is being used to return several different datatypes,
type coersion may be effective:
.s
.nf
	int		call();
	long		myfun();
	long		larg;
	long		result;
.s
	result = (long)call(myfun, 1, _&larg);
.s
.i -8;Bugs:
.s
.fill
WARNING: watch out for trouble if the PDP-11 routine does
any I/O.  Floating-point data return has not been implemented.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########callc###Call (Fortran) subroutine from C
.pg
.hl 1 ^&Call (Fortran) subroutine from C\&
.s 2
.c ;*********
.c ;* callc *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
integer	callc
.s
ivalue = callc(routine, arg1, ... argN)
.s
.i -8;Description
.s
.fill
User-callable interface between Fortran programs and
subroutines written in C.
.s
The arguments are as follows:
.s
.nf
	routine		The name of the C routine to  be
			called.
.s
	arg1		The   (location  of  the)  first
			argument.
.s
.fill
For example:
.s
.nf
	call callc(sort, table, nelement)
.s
.i -8;Bugs
.s
.fill
There will be I/O library conflicts since
C files have not been initialized.
.s
The following problem will occur if a C subroutine is
called from a Fortran main program:
.s
The C subroutine refers to the save register routine (CSV$)
which refers to the C main program.  This pulls in a large
amount of I/O dependent code, which may cause conflicts.
.s
What is worse is that there will be two main programs.
.s
It is therefore recommended that there be a C main program
that calls Fortran subroutines that may, in turn, call
C subroutines.
.s
The programmer can also define a dummy program that resolves
the globals referenced by CSV$ or modify callc.mac to provide
the necessary globals.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########caller##Return name of profiled caller
.pg
.hl 1 ^&Return name of profiled caller\&
.s 2
.c ;**********
.c ;* caller *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char	*
caller();
.s
.i -8;Description
.s
.fill
If the routine which called the current function was compiled
using the profile option, caller() returns a pointer to the
name of the calling function.  If the calling function was
not compiled with profiling enabled, a pointer to the null
string is returned.
.s
Example:
.s
.nf
	foo()
	{
		foobar();
	}
.s
	foobar()
	{
		extern char *caller();
.s
		printf("foobar was called by _%s_\n",
				caller());
	}
.s
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########calloc##Allocate and Initialize Memory
.pg
.hl 1 ^&Allocate and Initialize Memory\&
.s 2
.c ;**********
.c ;* calloc *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
calloc(n, m);		/* NULL if no space	*/
int		n;	/* Number of elements	*/
int		m;	/* Size of an element	*/
.s
.i -8;Description
.s
.fill
Calloc() allocates space for n elements, each m bytes long.
On return, the space is zero'ed.  The program image will
be expanded as needed.
.s
If the requested space cannot be allocated, calloc() returns NULL.
.s
See also the description of malloc(), realloc() and sbrk().
.s
The space is returned by calling free().
.s
.i -8;Diagnostics
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########calltr##Trace Path to the Caller
.pg
.hl 1 ^&Trace Path to the Caller\&
.s 2
.c ;**********
.c ;* calltr *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
calltrace(outfd);
FILE		*outfd;
.s
.i -8;Description
.s
.fill
calltrace() prints the calling path (from main()) to the
routine that called calltrace()) on the indicated file.
The path is printed on one line as:
.s
.nf
	[ main sub1 sub2 ]
.s
.fill
If a subroutine within the path was not compiled with
profiling, or does not start with a call to the C register
register save routine, it will be printed as <nnnnnn>
(where <nnnnnn> is the octal address of the entry to the routine).
Note that the last entry on the line is the routine that
invoked calltrace().
.s
A call trace will be printed on exit if profiling is enabled
and the program aborts or calls error().
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########clearerr########Clear file error flags
.pg
.hl 1 ^&Clear file error flags\&
.s 2
.c ;************
.c ;* clearerr *
.c ;************
.s 2
.lm +8.nf
.i -8;Usage
.s
clearerr(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Clear the error flags for an open file.  This may be needed
for interaction with the user's console terminal.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########concat##Concatenate strings
.pg
.hl 1 ^&Concatenate strings\&
.s 2
.c ;**********
.c ;* concat *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
concat(out, in0, in1, ..., inN, 0);
char		*out;
char		*in0, in1, ... inN;
.s
.i -8;Description
.s
.fill
Concat concatenates the argument strings.  It returns a
pointer to the first byte of out.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########copy####Copy a given number of bytes
.pg
.hl 1 ^&Copy a given number of bytes\&
.s 2
.c ;********
.c ;* copy *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
copy(out, in, nbytes)
char		*out;	/* Output vector	*/
char		*in;	/* Input vector		*/
unsigned int	count;	/* Bytes to copy	*/
.s
.i -8;Description
.s
.fill
Copy the indicated number of bytes from the input area
to the output area.  Return a pointer to the first free
byte in the output area.
.s
The copying will be faster if out and in are either both
even or both odd addresses.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########cpystr##String copy
.pg
.hl 1 ^&String copy\&
.s 2
.c ;**********
.c ;* cpystr *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
cpystr(out, in);
char		*out;
char		*in;
.s
.i -8;Description
.s
.fill
Copy "in" to "out".  Return a pointer to the end of out.
This allows the following usage:
.s
.nf
	char		*ptr;
	char		buffer[BUFSIZ];
	extern char	*cpystr();
.s
	ptr = cpystr(buffer, text);
	ptr = cpystr(ptr, more__text);
.s
.fill
The above sequence appends more__text to the copy of text
already moved into the buffer.
.s
.i -8;Bugs
.s
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########csv$####C register save and restore
.pg
.hl 1 ^&C register save and restore\&
.s 2
.c ;********
.c ;* csv$ *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
jsr	r5,csv$
_...
jmp	cret$
.s
.i -8;Description
.s
.i -8;##		C program Run-time Environment
.s
.fill
Each C subroutine starts with a call to CSV$ and exits by
jumping to CRET$.  Upon exit, the stack need not be equal
to its value on entrance.
.s
During the execution of all C subroutines, register R5
points to the current "environment."  Within a subroutine,
it appears as follows:
.s
.nf
	 ______________________________
	|		|
  SP ->	| 1st loc. var. | -10(R5)	C$AUTO-2(R5)
	|______________________________|
	|		|
	| Saved R2	|  -6(R5)
	|______________________________|
	|		|
	| Saved R3	|  -4(R5)
	|______________________________|
	|		|
	| Saved R4	|  -2(R5)
	|______________________________|
	|		|
  R5 ->	| Saved R5	|
	|______________________________|
	|		|
	| Return add.	|  +2(R5)
	|______________________________|
	|		|
	| First arg.	|  +4(R5)	C$PMTR+0(R5)
	|______________________________|
	|		|
	| Second arg.	|  +6(R5)	C$PMTR+2(R5)
	|______________________________|
.s
.fill
Within a subroutine, Registers R0-R4 and the top of the
stack, (sp) are available for use. Registers R0 and R1
are not preserved by subroutines and may be used to pass
a return value.
.s
R5 must not be modified by a subroutine.  All variable
addressing must be done by offsets to R5.  Subroutine
arguments must be accessed by reference to C$PMTR.
Subroutine local variables must be accessed by reference
to C$AUTO.  This permits modification of calling sequences
without rewriting all subroutines.
.s
CSV$ refers to global symbol $$main to call the run-time startup
program from the (RSX) library.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ctime###Convert time value to ascii
.pg
.hl 1 ^&Convert time value to ascii\&
.s 2
.c ;*********
.c ;* ctime *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
_#include <time.h>
.s
char *
ctime(tvecp)
long		*tvec;	/* Time value pointer	*/
.s
char *
asctime(tm)
struct tm	*tm;	/* Time buffer pointer	*/
.s
.i -8;Description
.s
.fill
ctime() converts a time value, as returned by time()
to an ascii string.  For compatibility with previous
versions, ctime(0) obtains and converts the current
time of day.  Note, however, that ctime(0) is not
portable.
.s
asctime() converts a time vector, as returned by localtime()
to an ascii string.
.s
The string is statically allocated.  It has the format
.s
.nf
	Sun Sep 16 01:03:52 1973_\n_\0
	012345678901234567890123 4 5
	0         1         2
.s
.fill
All the fields have constant width.  To remove the trailing
newline, just:
.s
.nf
	char		*tp;
	extern char	*ctime();
	int		tvec[2];
.s
	time(_&tvec);		/* Get time	*/
	tp = ctime(_&tvec);	/* in Ascii	*/
	tp[24] = '_\0';		/* Fix newline	*/
.s
.i -8;Bugs
.s
.fill
There is no range checking on the information passed.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########delete##Delete a named file
.pg
.hl 1 ^&Delete a named file\&
.s 2
.c ;**********
.c ;* delete *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
delete(filename)
char		*filename;
.s
.i -8;Description
.s
.s
.fill
Delete the named file.
Returns 1 on success, 0 on error (such as file not found).
If an error occurs, $$ferr can be checked for the error code.
.s
On RSX modes, the filename as passed need not include an explicit
version number.  If none is provided, file.nam;0 will
be deleted. Note that the filename returned by fgetname()
always includes an explicit version.
.s
On RT modes, the only error ever returned is "File not found".
.s
.i -8;Bugs
.s
On RSX/VMS, the file must be in the user's current ("SET DEFAULT")
directory.
.s
On RSTS, "SY0:" cannot be distinguished from "SY:".
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########envsave#Multi-level function return
.pg
.hl 1 ^&Multi-level function return\&
.s 2
.c ;***********
.c ;* envsave *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
int *
envsave();
.s
envreset(frame, hval, lval)
int		*frame;	/* Frame pointer	*/
int		hval;	/* High-order return	*/
int		lval;	/* Low-order return	*/
.s
.i -8;Description
.s
.fill
These functions permit a multi-level return from a C function
to one of its dynamic ancestors in the stack.
.s
Envsave() returns the address of the frame of its
caller.  Later calls to envreset() with this pointer as an
argument will create the illusion that the last function invoked
by the caller of envsave() returned directly with a value
equal to the parameter(s) to envreset().
.s
Envreset() simulates a return to the function whose frame
pointer matches frame (the result of a previous call to
envsave()).  Envreset() may be made to look as if it is
returning a one or two word value.  If only a single integer
value is passed in addition to envreset(), it looks as if
a 1-word value was returned.  If two integers (or equivalently,
a long) are specified, it looks like a long is returned.
Thus any 1 or 2 word value may be returned because of the lax
type checking in C.
.s
It is catastrophic to attempt to return to a function
which has itself already returned!!!
.s
The following illustrates use of envsave() and envreset():
.s
.nf
	int	*save__frame;
.s
	process() {
	  ...
	  save__frame = envsave();
	  if (subroutine()) {
	    /*
	     * Success
	     */
	  }
	  else {
	    /*
	     * Failure
	     */
	  }
	}
.s
	subroutine() {
	  /*
	   * Abort on error
	   */
	  if (...)
	    envreset(save__frame, 0)
	  /*
	   * Success
	   */
	  return(1);
	}
.s
.fill
When subroutine() detects an error, it calls envreset() to
return to the caller of process(), passing a "failure" flag.
.s
.i -8;Bugs
.s
If the caller of envsave() has already returned, disaster will
result.
.s
It is highly recommended that programs use setjmp/longjmp instead.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########error###Error exit from C programs
.pg
.hl 1 ^&Error exit from C programs\&
.s 2
.c ;*********
.c ;* error *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
error(format, arglist);	/* Fatal error exit	*/
.s
.i -8;Description
.s
.fill
An error message is written to stderr and the program exits.
If profiling was enabled, a subroutine call trace will be
written to stderr.
.s
The exit status, $$exst, will be set to "error".
.s
.i -8;Diagnostics
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########exit####Exit from C programs
.pg
.hl 1 ^&Exit from C programs\&
.s 2
.c ;********
.c ;* exit *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
exit(status);		/* Exit, no return	*/
.s
exits(status);		/* Exit with status	*/
.s
$$fail();		/* Immediate exit	*/
.s
extern int $$exst;	/* Exit status value	*/
.s
.i -8;Description
.s
.fill
On exit, the following sequence occurs:
.nf
  1.	The user-supplied wrapup() function is called.
  2.	Profiles (if requested) are printed.
  3.	All files are closed.
  4.	The program exits to the operating system.
.s
.fill
By calling exits() with an appropriate value, the program can
pass an exit status code to the operating system monitor.
The RT11 and RSX11 monitors define exit status codes differently;
the value passed to exits() is not checked for consistancy.
.s
.nf
  RT11	RSX	Meaning
    1.	 1.	IO__SUCCESS -- no message is printed
    2.	 0.	IO__WARNING -- minor error
    4.	 2.	IO__ERROR   -- serious error
    8.	 4.	IO__FATAL   -- severe error
.s
.fill
Calling exit() with some other value is equivalent to
calling exits(IO__SUCCESS).  This is compatible with
Unix programs that exit by calling exit() without parameters.
.s
Note that the program may also set a value into $$exst
and exit via exit():
.s
.nf
	extern int $$exst;
.s
	...
.s
	$$exst = IO__WARNING;
	exit();
.s
.fill
If the program calls, or jumps to $$fail, it will immediately
exit to the operating system without closing files, calling
wrapup(), or writing a profile file.
.s
.i -8;Diagnostics
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fabs####Floating absolute value
.pg
.hl 1 ^&Floating absolute value\&
.s 2
.c ;********
.c ;* fabs *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
double
fabs(val)
double		val;
.s
.i -8;Description
.s
.fill
Return absolute value of a floating argument.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fclear##Clear file error flags
.pg
.hl 1 ^&Clear file error flags\&
.s 2
.c ;**********
.c ;* fclear *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
fclear(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Clear the error flags for an open file.  This may be needed
for interaction with the user's console terminal.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fclose##Close a currently-open file
.pg
.hl 1 ^&Close a currently-open file\&
.s 2
.c ;**********
.c ;* fclose *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
fclose(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Close the file.  Returns 0 if ok, -1 if an error.
The error code is saved in $$ferr.
.s
Note that stderr is never closed.  On RT11-mode, the
last block of a disk file which was open for output
is filled with nulls.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########feof####Test for end of file
.pg
.hl 1 ^&Test for end of file\&
.s 2
.c ;********
.c ;* feof *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
feof(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Return 1 if end of file on iop, else zero.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ferr####Test for file error
.pg
.hl 1 ^&Test for file error\&
.s 2
.c ;********
.c ;* ferr *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
ferr(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Return 1 if an error has been seen on the indicated file.
.s
.i -8;Bugs
.s
Obsolete, use ferror() instead.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ferror##Test for file error
.pg
.hl 1 ^&Test for file error\&
.s 2
.c ;**********
.c ;* ferror *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
ferror(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Return 1 if an error has been seen on the indicated file.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fflush##Flush output buffers
.pg
.hl 1 ^&Flush output buffers\&
.s 2
.c ;**********
.c ;* fflush *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
fflush(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Flush output buffers.  This routine actually does I/O.
$$ferr is set if anything unexpected happens.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fget####Input a binary record
.pg
.hl 1 ^&Input a binary record\&
.s 2
.c ;********
.c ;* fget *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
fget(buffer, maxbytes, iop);
char		*buffer;
int		maxbytes;
FILE		*iop;
.s
.i -8;Description
.s
.fill
Read a record from the indicated file.  Maxbytes is the
maximum record size.
.s
On RSX, the record is read by a direct call to the file
service GET$ routine.  Your program must not mix calls
to fget() with calls to other I/O routines, such as
fgets().
.s
On RT11, the file must have been written by fput().
.s
The actual number of bytes read is returned.  Use feof()
to test for end of file.
.s
Fget() is not present on Unix standard I/O libraries.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fgetname########Convert file name to Ascii
.pg
.hl 1 ^&Convert file name to Ascii\&
.s 2
.c ;************
.c ;* fgetname *
.c ;************
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
fgetname(iop, buffer);
FILE		*iop;
char		*buffer;
.s
.i -8;Description
.s
.fill
Convert the file name to Ascii and put it in the buffer.
.s
On native RSX (and RSTS/E emulation) the device name,
file name, file type, and version are converted to Ascii.  The UIC
is converted if it was specified and differs from the job's
default UIC.  (Note that on VMS compatibility mode, the
UIC is unavailable.)  The version is converted to octal on
native RSX and to decimal on VMS emulation.  The result should
be sufficient to delete the file.
.s
On RT11 modes, the file name passed to fopen() is copied to
the buffer without further processing.  If the file was opened
by an fwild()/fnext() sequence, the correct file name is
returned.  (On RSTS/E, the PPN is returned as well.)
.s
Fgetname() returns a pointer to the buffer argument.
.s
Fgetname() is present in the Vax-11 C support library, but
is not generally present on Unix implementations.
.s
Note that fgetname() returns a fully-qualified file specification
including, where possible, the disk, directory, and version
of the file.  If only the actual file name is needed, the
following code segment may be executed.  As shown, it writes
the file name and filetype (extension) to a global "buffer".
.s
.nf
    getfilename(fd, buffer)
    FILE		*fd;
    register char	*buffer;
    {
	register char *tp;
	register char c;
.s
	fgetname(fd, buffer);
	/*
	 * Skip over node and device name
	 */
	while ((tp = strchr(buffer, ':')) != NULL)
	    strcpy(buffer, tp + 1);
	/*
	 * Skip over [UIC] or [PPN] if present
	 */
	c = EOS;
	switch (*tp) {
	case '[':	c = ']'; break;
	case '(':	c = ')'; break;
	case '<':	c = '>'; break;
	}
	if (c != EOS
	 _&_& (tp = strchr(buffer, c)) != NULL) {
	    strcpy(buffer, tp + 1);
	/*
	 * Don't include version
	 */
	if ((tp = strchr(buffer, ';')) != NULL)
	    *tp = EOS;
    }
.s
.i -8;Bugs
.s
.fill
Various operating systems behave differently.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fgets###Read a string from a file
.pg
.hl 1 ^&Read a string from a file\&
.s 2
.c ;*********
.c ;* fgets *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
fgets(buffer, maxbytes, iop);
char		*buffer;
int		maxbytes;
FILE		*iop;
.s
char *
fgetss(buffer, maxbytes, iop);
char		*buffer;
int		maxbytes;
FILE		*iop;
.s
.i -8;Description
.s
.fill
Fgets() reads a string into the buffer from the indicated file.
Maxbytes is the maximum number of bytes to read.
The string is terminated by a newline character,
which is followed by a null.  Fgets() normally returns buffer.
It returns NULL on end of file or error.  If the line being read
is maxbytes long or longer, no newline is appended.
.s
Fgetss() is identical to fgets() except that the terminating
newline is replaced by a null.  (This is compatible with gets(),
but fgetss() is not present in the Unix standard library.)
.s
Note that fgets keeps the newline, while fgetss() and gets()
delete it. To delete the newline after executing fgets(),
execute the following:
.s
.nf
	buffer[strlen(buffer) - 1] = 0;
.s
.i -8;Bugs
.s
.fill
Note that fgets() returns the next logical text line.
Certain RSX-11 or RMS file formats, such as Runoff output files
or task-builder load maps, often have multiple text lines
packed into one logical record.  Fgets() processes these correctly.
Note, however, that the record location, as returned by
ftell(), is the location of the physical record, as returned
by the operating system.  The "obvious" ftell/fgets sequence,
.s
.nf
	rfa = ftell(fd);
	fgets(text, sizeof text, fd);
.s
.fill
will not work properly if multiple text lines are packed
into one RSX-11 record.
.s
These routines do not understand some of the more baroque
RSX-11 file formats, such as "Fortran carriage control"
or VMS print-file format.  The typeout program in the
tools package, T.C, shows an example of complete record
deblocking and "seek to byte" procedure.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fileno##Get logical unit number
.pg
.hl 1 ^&Get logical unit number\&
.s 2
.c ;**********
.c ;* fileno *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
fileno(iop)
FILE		*iop;
.s
.i -8;Description
.s
.fill
Return the logical unit number associated with the file.
(On RT11, the channel number is returned.)
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fill####Fill a Block of Memory With a Character
.pg
.hl 1 ^&Fill a Block of Memory With a Character\&
.s 2
.c ;********
.c ;* fill *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
fill(addr, ch, count);
char		*addr;
char		ch;
unsigned int	count;
.s
.i -8;Description:
.s
.fill
Fill the count characters of memory beginning at addr with the
character ch.
.s
Note:  To fill with zero bytes, you can use zero(), which is slightly
faster.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########flun####Get logical unit number
.pg
.hl 1 ^&Get logical unit number\&
.s 2
.c ;********
.c ;* flun *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
flun(iop)
FILE		*iop;
.s
.i -8;Description
.s
.fill
Return the logical unit number associated with the file.
(On RT11, the channel number is returned.)
.s
.i -8;Bugs
.s
Obsolete -- use fileno() for transportability.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fmkdl###Mark file for deletion -- obsolete
.pg
.hl 1 ^&Mark file for deletion -- obsolete\&
.s 2
.c ;*********
.c ;* fmkdl *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
fmkdl(iop);
FILE		*iop;
.s
.i -8;Description
.s
.s
.fill
fmkdl() closes and deletes the specified file.
Returns 0 on success, -1 on error.
.s
This routine is obsolete.  New programs should call
delete() instead.  To delete an existing file, the program
may execute:
.s
.nf
	char	buffer[80];
	...
	fgetname(iop, buffer);
	fclose(iop);
	delete(buffer);
.s
.i -8;Bugs
.s
.fill
On RT11, the job is aborted with an error message if the
file wouldn't parse.  $$ferr is set to "file not found"
if the program tries to delete something (like the line-printer)
that it shouldn't.  Note that RT11 does not flush the
last buffer before deleting the file.
.s
On VMS compatibility mode, the file must be in the user's
default directory.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fopen###C library file opener
.pg
.hl 1 ^&C library file opener\&
.s 2
.c ;*********
.c ;* fopen *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
FILE *
fopen(name, mode);
char		*name;	/* File to open 	*/
char		*mode;	/* Open modes		*/
.s
FILE *
freopen(name, mode, iop);
char		*name;	/* File to open		*/
char		*mode;	/* Open modes		*/
FILE		*iop;	/* I/O pointer		*/
.s
.i -8;Description
.s
.fill
Fopen opens a new or existing file in the indicated mode:
.s
.nf
	r	Read the existing file sequentially
	w	Create and write the file sequentially
	a	Append to the file
	n	Not record oriented
	u	RSX-mode "unbuffered i/o"
		RT11-mode: use .ttyin and .ttyout
.s
.s
.fill
Either "r", "w", or "a" must be given.  "n" and "u" are
optional.  "n" should be given for "binary" files.
Note that "n" mode will create fixed-block records on
RSX systems. Append mode does not work on native RT11.
.s
Note that "n" and "u" are not compatible with other
Unix systems.
.s
.i -8;Implementation Details
.s
On RSX, "u" mode files will be created with the
"variable-length" attribute.  On RSTS/RSX emulation, text
files (neither "n" nor "u" specified) will be created with
"stream" attribute.
.s
On RSX, if the record type bits in the record attribute byte
(F.RATT in the FDB) is zero, the file will be read as if
the "n" was specified.  Note that, if the file contains
carriage-return line-feed sequences, the entire sequence
will be passed to the user's program.  If record attributes
are understandable, the carriage-return will be deleted
from <CR><LF> sequences.
.s
.i -8;##	On RSX, if the "file" is being opened for write to the same
device/unit
as stderr (the user's "command console), the character stream is
diverted to stderr.  This avoids synchronization problems by
funnelling all output through the same buffer.  In
addition, output to any terminal device is done via QIO's to
the terminal, IO.WLB for normal mode, IO.WAL for "n" mode.
.s
On RT11, when opening a file for writing, a specific block
allocation may be included in the Ascii file specification
following standard RT-11 syntax: "DKn:file.nam[nnn]" where the
"nnn" specifies the number of blocks to allocate to the file.
After opening a file successfully, the actual file size (or
number of blocks allocated) will be found in (FILE *)fd->io__size.
.s
If the RT11 library decides that the file is really the user's
command terminal, single-character I/O will be performed (by
calling .ttyin and .ttyout).  Note that the "special-mode"
bits must be set in the Job Status Word by the program if it requires
true single-character or immediate return input. Output to the
terminal will be performed without buffering, which is useful
for screen updating, but otherwise expensive.
.s
Fopen() returns NULL on errors -- $$ferr gets an error code.
On RT11, this will be a RSTS/E compatible code (described in
iov), while on RSX, this will be the FCS error code.
.s
On RT11, the file name pointed to by the "io__name" field of
the iov is either the file name string as passed to fopen()
or an ascii string reconstructed from the 4-word Rad50
device block if the file was opened as part of a fwild/fnext
sequence.  By saving the ascii string, RSTS/E is able to
re-parse logical device names and PPN's, which are not present
on native RT11.
.s
On VMS compatibility mode, the device and directory of the
file name argument are saved in the iov "io__dname"
field for use by fgetname().
.s
Note that "no buffer space available" (IE.NBF or E$$NSP) and
"invalid lun" (IE.ILU or E$$NOC) may be generated by fopen.
On RT11, if the file cannot be opened because the user's program
has already opened the channel, an E$$ILU error will be returned.
.s
The same file may not be used for both reading and writing
except if the program writes a disk file, then repositions
and reads it using ftell()/fseek().  In this case, the program
should call rewind() or freopen() to reinitialize the file before
using fseek().
.s
Except in the one specific case of the RT11 console terminal
open in "u" mode, an open file must not be used for reading
and writing at the same time.
.s
Freopen() substitutes the named file in place of the open
file -- indicated by iop.  The file currently open on
iop is closed.  Freopen returns iop.  If the open failed,
iop will be deallocated.  Note that freopen loses any pending
fwild/fnext status.
.s
.s
.i -8;Bugs
.s
Append mode cannot work on native RT11 given the design of the
RT11 file system.
.s
The RT11 file system does not support files greater than
65535 blocks long.
.s
RT11 does not get the actual keyboard name of the console
terminal, although this isn't too hard to do on RT11/RSTS/E.
.s
Freopen() cannot be used to assign a file to stderr as there
is code throughout the i/o package for special treatment of
stderr.  For example, it cannot be closed by a user-written
program.
.s
In RSX modes, the maximum number of files that may be simultaneously
open is defined at assembly time by a macro (FSRSZ$) which is
expanded when fopen.mac is assembled.  The default FSRSZ$
parameter is 4.  This may be modified by using the task builder
/ACTFIL=n option.  The default FSRSZ$ value may be specified
when the RSX library is built by editing assembly parameter
N$$FIL in RSX.MAC.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fprintf#Formatted Output Conversion
.pg
.hl 1 ^&Formatted Output Conversion\&
.s 2
.c ;***********
.c ;* fprintf *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
fprintf(iov, format, arg1, ...);
FILE		*iov;
char		*buffer;
char		*format;
.s
.i -8;Description
.s
.fill
fprintf() converts and formats its arguments, writing the
result to the indicated file.
.s
For information on formatting, please refer to the description
of printf.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fps#####Set floating point status (FPS)
.pg
.hl 1 ^&Set floating point status (FPS)\&
.s 2
.c ;*******
.c ;* fps *
.c ;*******
.s 2
.lm +8.nf
.i -8;Usage
.s
_#include <fps.h>
.s
fps(status)
unsigned int	status;
.s
.i -8;Description
.s
.fill
This routien sets the floating point status by simply executing
the LDFPS instruction.  This allows a user program to switch
between roundoff and truncation modes or enable/disable
floating point interrupts.  Bit definitions are in the
include file, "fps.h".
.s
For example, to enable rounding and all fpu error interrupts
except underflow, do the following:
.s
.nf
	_#include <fps.h>
	fps(FIUV | FIV | FIC);
.s
.i -8;Bugs
.s
.fill
The C library doesn't initialize the floating point status.
This routine -- with appropriate parameters -- should be
in all C main programs that require floating point operations.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fput####Output a binary record
.pg
.hl 1 ^&Output a binary record\&
.s 2
.c ;********
.c ;* fput *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
fput(buffer, nbytes, iop);
char		*buffer;
int		nbytes;
FILE		*iop;
.s
.i -8;Description
.s
.fill
The specified record is written to the file.  The file
must have been opened 'n' so newlines
aren't stuffed here and there.
.s
On RT11, the file is only readable by fget.
.s
Nbytes is always returned.
.s
A call to ferr() after calling fput() is advised.
.s
.i -8;Bugs
.s
On RT11, file close zeros the current output block.  To
prevent fget() from reading "zero length" records, fput
writes "record count + 1" on the file.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fread###Input a record
.pg
.hl 1 ^&Input a record\&
.s 2
.c ;*********
.c ;* fread *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
fread(object, sizeof (*object), nitem, iop)
int		nitem;
FILE		*iop;
.s
.i -8;Description
.s
.fill
Object points to a buffer that is to receive a specified
number of items from the file.
.s
fread() returns the actual number of items read.  This will
be zero if an end-of-file or error was encountered.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########frec####Return True if record-oriented file
.pg
.hl 1 ^&Return True if record-oriented file\&
.s 2
.c ;********
.c ;* frec *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
frec(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Return 1 if the file is record-oriented.
Note: in this context, record-oriented files are
not file-structured disks, nor are they the user's
console terminal.  Other terminals, along with devices
such as line-printers, qualify, however.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fscanf##Formatted input conversion
.pg
.hl 1 ^&Formatted input conversion\&
.s 2
.c ;**********
.c ;* fscanf *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
fscanf(fd, fmt, pointer(s))
FILE		*fd;	/* Input file pointer	*/
char		*fmt;	/* Format string	*/
.s
.i -8;Description
.s
.fill
Fscanf() parses the input file according to the indicated
format descriptor, storing the results in the pointer arguments.
It returns the number of successfully assigned input items.
.s
See the description of scanf() for further documentation.
.s
.i -8;Diagnostics
.s
Fscanf() returns -1 if an end of file condition
exists and no data was stored.  It returns -2 if a palpably
incorrect format, such as "_%" is encountered.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fseek###Reposition file pointer (seek)
.pg
.hl 1 ^&Reposition file pointer (seek)\&
.s 2
.c ;*********
.c ;* fseek *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
fseek(iop, offset, param);
FILE		*iop;	/* What device to seek	*/
long		offset;	/* New read position	*/
int		param;	/* Zero for abs. seek	*/
.s
.i -8;Description
.s
.fill
fseek() moves the file pointer to the indicated position.
The position must have been returned by ftell() or equal
zero for rewind.  Param must be zero.  The file/device must
be seekable.
.s
fseek() returns zero if correct, EOF if an error occurs.
If no error occurred, error and eof flags are reset.
.s
flseek() is an alternate entry with identical parameters
and actions.
.s
Note that on RSX, fseek() can be used to open a file for
update by a sequence such as:
.s
.nf
	fd = fopen("file.nam", "a");
	fseek(fd, 0L, 0);	/* Rewind file	*/
.s
.fill
If the file was opened on RSX using the 'n' mode switch, it will
be opened using file attributes "fixed 512-byte records with
no carriage control".  The offset pointer is thus
a virtual byte number and fseek may be used to freely reposition
the file pointer.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fspool##Spool file to default print queue
.pg
.hl 1 ^&Spool file to default print queue\&
.s 2
.c ;**********
.c ;* fspool *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
fspool(fp);
FILE		*fp;	/* Open file IOV	*/
.s
.i -8;Description
.s
.fill
This routine is called in lieu of fclose() when it is
desired to spool the file to the system line printer.
There is no control over the disposition or printout
parameters.
.s
Fspool() returns zero if the file was spooled,  it returns
an error code if spooling could not be initiated.
If called on native RT11, the file will be closed and
an "illegal device error" (E$$NOD = 6) will be returned.
.s
If the program needs access to the power of the queue
manager print spooler on native RSX, the RSX extensions library
spwn() routine should be used to spawn a command line to MCR after
closing the file as usual.
.s
If the program needs access to the power of the queue manager
on RSTS/E, it should close the file and use the routines in
the RSTS/E extensions library to execute the UU.SPL system
function call.
.s
On RSTS/E V7.0, the file is spooled to any line printer (RSX mode)
or to LP0: in RT11 mode.  The particular line printer cannot
be selected without modifying the source of fspool().
.s
If the file was opened by calling fwild()/fnext(), internal
buffers will not be freed, allowing the program to call
fnext() for subsequent files.
.s
.i -8;Bugs
.s
On RSTS/E RT11 mode, error codes will follow the description of the
spooling system function call.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ftell###Get file position for subsequent seek
.pg
.hl 1 ^&Get file position for subsequent seek\&
.s 2
.c ;*********
.c ;* ftell *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
long
ftell(iop);
FILE		*iop;	/* What device to seek	*/
.s
.i -8;Description
.s
.fill
ftell() returns the position of the read/write pointer of
the indicated file.  This value may be fed back to the
file system by calling fseek().  Note that the value is
a pointer to the record, not a block or byte pointer.  On RSX,
the program should flush the current record before calling ftell().
(Flush() is a noop on RT11.)
.s
If reading lines of text, the correct sequence is:
.s
.nf
	position = ftell(fd);
	if (fgets(buffer, sizeof buffer, fd) != EOF) {
		/*
		 * 'position' locates the record
		 * read by the call to fgets()
		 */
	}
.s
.fill
Make sure you declare ftell()
.s
.nf
	extern long ftell();
.s
.fill
If you do not, it will return garbage.
.s
.i -8;Bugs
.s
On both systems, the value returned is the position of the file
pointer (RFA), as a byte offset from the start of the file.
Note, however, that on RSX systems the pointer is to the start
of the current record.  It is not necessarily the case that
the start of a text line equates to the start of a record.
RSX supports many file record formats, including Fortran,
print__file and "unformatted" (with embedded control
information).  The latter files may have multiple text lines
embedded in each record.  This is handled internally by
the formatted read routines (i.e., fgets() and getc()).
On RSX, the only way to be certain of the exact record/line
correspondance is to use the fget() and fput() functions.
The T utility program (in the tools library) shows another
method of handling this problem.
.s
On RSTS/E RT11 mode, ftell will not process "large" files
(with more than 65535 blocks) correctly.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ftime###Compute time of day (augmented)
.pg
.hl 1 ^&Compute time of day (augmented)\&
.s 2
.c ;*********
.c ;* ftime *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
.s
_#ifdef unix
_#include <sys/types.h>
_#include <sys/timeb.h>
_#else
_#include <timeb.h>
_#endif
.s
long
ftime(buf);
struct timeb	*buf;
.s
.i -8;Description
.s
.fill
ftime returns the time of day in seconds.  The time buffer
receives the time of day, milliseconds since the current
second, and timezond and Daylight Savings time indicators.
.s
The time buffer has the following format:
.s
.nf
  struct timeb {
    long      time;	/* Time of day		*/
    unsigned  millitm;	/* Milliseconds		*/
    short     timezone;	/* Current timezone	*/
    short     dstflag;	/* Daylight Savings	*/
.fill
};
.s
The time value is the time since midnite, Jan 1, 1970, measured
in seconds.
.s
Note that, on Unix, the time buffer definition is stored
in the sys directory, while for Decus-C and Vax-11 C, it is
stored in the default system library (C: or sys$library:
respectively).
.s
.i -8;Bugs
.s
The Unix time function returns GMT.  This function returns
local time.  Timezone and dstflag are set to zero.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ftty####Test if terminal file
.pg
.hl 1 ^&Test if terminal file\&
.s 2
.c ;********
.c ;* ftty *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
ftty(iop)
FILE		*iop;
.s
.i -8;Description
.s
.fill
Return 1 if the file is a terminal-type device.
In general, this means the user's command terminal.
.s
.i -8;Bugs
.s
Obsolete -- use isatty(fileno(iop)) instead for transportability.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fwild###Wild-card file open
.pg
.hl 1 ^&Wild-card file open\&
.s 2
.c ;*********
.c ;* fwild *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
FILE *
fwild(name, mode);
char		*name;	/* File to open 	*/
char		*mode;	/* Open modes		*/
.s
FILE *
fnext(iop);
FILE		*iop;	/* I/O pointer		*/
.s
.i -8;Description
.s
.fill
Fwild() opens a new or existing file (whose file name may
contain "wild-cards").  Open modes are identical to those
given in fopen().  On return, the file name has been parsed,
but no file has yet been opened.  A NULL return means that
the file name did not parse correctly.
.s
Fnext() opens the first or next file which was defined
by a previous call to fwild().  If fnext() returns NULL, there
are no (more) files that match the wild-card specification.
.s
fwild/fnext handle RSX file version numbers correctly on
VMS compatibility mode (which uses the ODS2 disk structure).
Fwild/fnext do not handle version numbers correctly on
native RSX systems which use the FILES-11 (ODS1) disk structure.
For example, a program can request "foo.*;3", "foo.*;*", "foo.*;0",
and "foo.*;-1". Omitting a version number "foo.*" is equivalent
to "foo.*;0". Note that version number 0 means the "newest" file,
while version number -1 means the oldest.  (Version numbers are
not used on RT11 or RSTS/E.)
.s
For native RSX systems (using the FILES-11 disk structure), an
explicit version number and the wildcard version number work
correctly.  Version numbers 0 and -1 work only if the directory has
been reorganized by using the SRD utility program.  If the directory
has not been reorganized (such that the youngest version appears
first in the directory), fnext() will yield unpredictable results.
.s
On RT-11, the wildcard filename match is handled internally
to fwild/fnext. The parser will handle several forms of wild
file specs, including imbedded '*' and the single character
wildcard '_%', and acts the same as the DIRECTORY wildcard handler.
For convenience, a '?' acts the same as a '_%' in a match string.
.s
Note:  if a program executes fclose(), all file name information
will be lost.  The following sequence illustrates proper use
of fwild()/fnext():
.s
.nf
	if (gets(name__buff) == NULL)
		exit();
	if ((fd = fwild(name__buff, "r")) == NULL)
		error("Can't open _%s_\n", name__buff);
	for (count = 0; fnext(fd) != NULL; count++) {
		/*
		 * Process each file
		 */
		while (fgets(buffer, sizeof buffer, fd)
				!= NULL) {
			/*
			 * Process each record
			 */
		}
	}
	/*
	 * fnext() fails; the channel is closed.
	 * count has the number of files processed.
	 */
	if (count == 0)
		error("No matching files found");
.s
.fill
The following summarizes the types of wild-card processing
available on the various implementations of the C support
library:
.s
.nf
   Environment	Supports
.s
   Native RSX	"*"  matches  any filename, filetype, or
		version number.  Version ;0 and ;-1  are
		supported on ODS2  systems.   UIC's  may
		not contain wildcards.
.s
   RSX/VMS	As above, note that  version  ;-1  means
		the "earliest"  version.   Note  warning
		below.  Directory identifiers may not be
		wildcarded.  VMS systems support ODS2.
.s
   RSX/RSTS	Uses RSTS/E  wildcard  conventions:  "*"
		replaces   filename  or  filetype.   "?"
		matches any character.  PPN's may not be
		wildcarded.   Version  numbers  are  not
		supported on RSTS/E.
.s
   Native RT11	"*" replaces  any  string,  "_%"  or  "?"
		match any non-blank character.
.s
   RT11/RSTS	Uses RSTS/E wildcard  conventions  noted
		above.
.s
.i -8;Bugs
.s
.fill
On native RSX systems using ODS1 (FILES-11) disk structures,
version numbers will be processed properly only if directories
have been sorted (by using the SRD utility program, for example).
If directories are not sorted, and fwild() is invoked with version
number ;0 or ;-1, it will yield unpredictable results.
.s
The command language scan (CSI1$) on VMS compatibility mode does
not parse version number -1 (because it has a different meaning
on native VMS) and fwild() will consequently fail.
.s
If you want the oldest version, fwild() should be invoked with
a file name of the type "foo.*" or "foo.*;0" and, before calling
fnext() for the first time, you should set the correct bits in
the IOV flag word as follows:
.s
.nf
	if ((fd = fwild(file, "r")) == NULL)
		error("can't open file _%s", file);
	if ((fd->io__flag _& IO__VER) != 0
			_&_& version__minus__1__wanted)
		fd->io__flag |= IO__VM1;
.s
.fill
Flag bit IO__VER signals "version 0 or -1", while bit IO__VM1
signals version minus 1.  Again, note that this must be done
before the first call to fnext().
.s
On native RT11 and all RSTS/E modes, fwild/fnext will fail
if the device is not directory structured (even if no wildcard
file is specified).  If this is a problem, you should write:
.s
.nf
	if ((fd = fwild(filename, mode)) != NULL)
		iswild = 1;
	else if ((fd = fopen(filename, mode) != NULL)
		iswild = 0;
	else error("cannot open the file");
.s
.fill
The program must then test iswild to determine if it
must call fnext() or if processing should be initiated
directly.
.s
On all RSTS/E modes, there may be problems with logical name
translation as file name strings must be parsed more than once.
Thus, if a programer defines a logical name which is identical
to a valid physical device name, a wildcard lookup may access
the wrong unit.  This problem is described in the RSTS/E
documentation.
.s
Fwild/fnext was designed to work only with disk devices.  It
will not necessarily work correctly with other directory-structured
devices, such as magtape.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########fwrite##Output a record
.pg
.hl 1 ^&Output a record\&
.s 2
.c ;**********
.c ;* fwrite *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
fwrite(object, sizeof (*object), nitem, iop)
int		nitem;
FILE		*iop;
.s
.i -8;Description
.s
.fill
Object points to a buffer that is contains a specified
number of items to be written to the file.
.s
fwrite() returns the actual number of items written. This will
be zero if an error was encountered.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########getchar#Get characters
.pg
.hl 1 ^&Get characters\&
.s 2
.c ;***********
.c ;* getchar *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
getchar();
.s
getc(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Getchar() reads one character from the standard input file.
getc() reads one character from the indicated input file.
.s
Both return EOF on error or end of file.
.s
To continue reading after EOF, the program may execute:
.s
.nf
	iop->io__flag _&= ~(IO__ERR | IO__EOF);
.s
.i -8;Bugs
.s
.fill
The "continue after EOF" sequence works on RT11/RSTS when reading
from the terminal.  It does not work on VMS compatibility mode
(subsequent reads continue to give EOF).
It has not been tested on other modes.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########gets####Read a string from stdin
.pg
.hl 1 ^&Read a string from stdin\&
.s 2
.c ;********
.c ;* gets *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
gets(buffer);
char		*buffer;
.s
.i -8;Description
.s
.fill
gets() reads a string into the buffer from the standard
input (stdin).  The string is terminated by a newline character,
which is replaced in the buffer by a null.  Gets() returns
buffer or NULL on end of file or error.
.s
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########gettty##Get control terminal name
.pg
.hl 1 ^&Get control terminal name\&
.s 2
.c ;**********
.c ;* gettty *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
gettty(buffer);
char		*buffer;
.s
.i -8;Description
.s
.fill
Store the device name of the control terminal in the buffer.
If this cannot be done, a null string will be returned.
.s
.i -8;Bugs
.s
On RSX modes, gettty() uses the GLUN$ executive directive
to obtain the device name and unit number of stderr (which
always is assigned to the control terminal).
.s
On RT11 modes, gettty() calls iovtoa to return the file name
on which stderr was opened.  This will not necessarily contain
a valid unit number.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########getw####Input a binary integer from a file
.pg
.hl 1 ^&Input a binary integer from a file\&
.s 2
.c ;********
.c ;* getw *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
getw(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
getw() reads one (16-bit) word from the indicated file.
The program must call feof() or ferr() to test for end
of file or error.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########inchr###Find Index of Character in a String
.pg
.hl 1 ^&Find Index of Character in a String\&
.s 2
.c ;*********
.c ;* inchr *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
inchr(stng, chr)
char		*stng;	/* String to search in	*/
char		chr;	/* Byte to search for	*/
.s
.i -8;Description
.s
.fill
If chr is in stng, return its index, i.e. the offset in bytes from
the beginning of the string - inchr("abc",'b') == 1.  If chr is not
in stng, return -1.
.s
See also strchr().
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########index###Find First Instance of a Character in a String
.pg
.hl 1 ^&Find First Instance of a Character in a String\&
.s 2
.c ;*********
.c ;* index *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
index(stng, chr)
char		*stng;	/* String to search in	*/
char		chr;	/* Byte to search for	*/
.s
.i -8;Description
.s
.fill
If chr is in stng, return a pointer to it.  If not,
return NULL.
.s
.i -8;Bugs
.s
Obsolete; use strchr() instead.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########iov#####I/O vector definition
.pg
.hl 1 ^&I/O vector definition\&
.s 2
.c ;*******
.c ;* iov *
.c ;*******
.s 2
.lm +8.nf
.i -8;Usage
.s
_#include <stdio.h>
.s
_... to be supplied ...
.s
.i -8;#####Bits in iov.io__flag:
.s
_#define __IOREAD 0000001	/* read 		*/
_#define __IOWRT  0000002	/* write		*/
_#define __IONBF	0000004	/* Unbuffered, "u" mode	*/
_#define __IOMYBUF 000010	/* I/O lib. owns buffer	*/
_#define __IOEOF	0000020	/* End of file seen	*/
_#define __IOERR	0000040	/* Error seen		*/
_#define __IOSTRG	0000100	/* For sprintf		*/
_#define __IORW	0000200	/* Open for read/write	*/
_#define	IO__BZY	0000400	/* Buffer write needed	*/
_#define	IO__APN	0001000	/* Append mode open	*/
_#define	IO__NOS	0002000	/* No newlines needed	*/
_#define IO__NEWL	0004000	/* RSX TTY newline hack	*/
_#define	IO__FIL	0010000	/* Disk file		*/
_#define	IO__TTY	0020000	/* Console terminal	*/
_#define	IO__REC	0040000	/* Record device	*/
_#define	IO__OPN	0100000	/* Open file		*/
_#define IO__EOR	(IO__ERR | IO__EOF)
.s
.i -8;#####Bits in iov.io__wflag:
_#define IO__WLD	0000001	/* fwild: wildcard file	*/
_#define IO__VM1	0000002	/* fwild: version ;-1	*/
_#define IO__VER	0000004	/* fwild: ;0 or ;-1	*/
_#define IO__WF1	0000010	/* fwild first flag	*/
_#define	IO__NLH	0000020	/* Newlines hack bit	*/
.s
.i -8;#####Bits in iov.io__rsflag (RSTS native only)
_#define	IO__ODT2	0100000	/* ODT mode (RSTS only)	*/
.s
extern int   $$ferr;	/* Error word		*/
extern FILE  *stdin;	/* Standard input  file	*/
extern FILE  *stdout;	/* Standard output file	*/
extern FILE  *stderr;	/* User's command tty	*/
extern int   $$exst;	/* Exit status		*/
.s
.i -8;Description
.s
.fill
Define the I/O vector structure used for communication by
all I/O routines in the C library.  Note that it is
different for RSX and RT11 modes.  Note also that certain bits
in IO__FLAG are only meaningful for one flavor of I/O.
.s
The RSX-mode IOV contains an entire file data block (FDB).
Also, 'io__uic' contains the binary UIC of the directory
via which the file is being accessed, not the 'owner' UIC.
It is this UIC which is given when fgetname() is called.
.s
The RT11-mode IOV contains only enough information to read and
write files plus a pointer to an Ascii string with the file
name argument to fopen().  The file name is needed for fgetname()
and to allow deleting files given the IOV pointer.
.s
The following files are defined here:
.s
.nf
	stdin	The standard input file.
.s
	stdout	The standard output file.
.s
	stderr	The error  output  file.   Note:  on RSX
		systems, stderr is opened on LUN 1.
.s
.fill
$$ferr (error word) is also defined here.  This is set non-zero
if an error occurred when performing I/O.  On RSX, the standard
I/O error code is returned.  On RT11, an error code compatible
with RSTS/E usage is returned:
.s
.nf
	Global   Value	Meaning
	E$$ILF	02. 002	Illegal file name
	E$$NOR	04. 004	No room for user on device
	E$$FNF	05. 005	Can't find file or account
	E$$NOD	06. 006	Not a valid device
	E$$ILU	07. 007 I/O channel in use
	E$$NOO	09. 011	I/O channel not open
	E$$EOF	11. 013	End of file on device
	E$$FAT	12. 014	Fatal system I/O failure
	E$$ERR	13. 015	User data error on device
	E$$FND	16. 020 File already found (protected)
	E$$NOC	17. 021	Too many open files on unit.
	E$$NSP	32. 040	No memory space for buffer
.s
.fill
E$$FAT (12) is set only if an "impossible" error occurs.  While this
may indicate a bug in the RT11 library, it is more likely to be
a user programming error (like passing garbage to an I/O routine).
.s
E$$ILU (7) is set if fopen tries to open a channel that is already
in use.
.s
The perror() library routine may be used to print an error message
defined for the error code.
.s
$$exst (exit status) is used to transmit a termination code to
the operating system.  The value is set by calling exit() or
exits().  The following exit status values are defined in stdio.h:
.s
.nf
  Global  _#define    RSX RT11 Meaning
  E$$XOK IO__SUCCESS   1    1  Normal
  E$$XWA IO__WARNING   0    2  Warning
  E$$XER IO__ERROR     2    4  Error
  E$$XFA IO__FATAL     4    8  Severe Error
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########irand###Random number modulus argument
.pg
.hl 1 ^&Random number modulus argument\&
.s 2
.c ;*********
.c ;* irand *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
irand(arg)
int		arg;
.s
.i -8;Description
.s
.fill
Generate a pseudorandom number in the range 0 .. (arg-1).
If arg is zero, generate a number in the range 0 .. 32767.
.s
Note that the algorithm is prone to nonrandom sequences when
considering the next pseudorandom number.
.s
irand is equivalent to the following sequence:
.s
.nf
	extern long	rand;
.s
	if (arg == 0) arg = 32767;
	return(((rand() >> 8) _& 32767) _% arg);
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isalloc#Check If a Pointer Points to Allocated Memory
.pg
.hl 1 ^&Check If a Pointer Points to Allocated Memory\&
.s 2
.c ;***********
.c ;* isalloc *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
isalloc(buff)
char *buff;
.s
.i -8;Description
.s
.fill
isalloc() returns 1 if buff is a pointer to a block of memory
allocated by calloc(), malloc(), or realloc() that has not been
freed (by free()); -1 if it is a freed allocated block; and 0
otherwise.
.s
isalloc(NULL) == 0.
.s
Note:  The test done by isalloc() is definitive, unlike the tests
done by, for example, free() and msize(), which will yield false
positives at least 25_% of the time.  However, isalloc() is slower.
.s
.i -8;Bugs
.s
Not portable.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isalnum#Return TRUE if Upper-case alphanumeric argument
.pg
.hl 1 ^&Return TRUE if Upper-case alphanumeric argument\&
.s 2
.c ;***********
.c ;* isalnum *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isalnum(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if c is an alphanumeric character, 0 otherwise.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isalpha#Test for alphabetic argument
.pg
.hl 1 ^&Test for alphabetic argument\&
.s 2
.c ;***********
.c ;* isalpha *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isalpha(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if c is an alphabetic character, 0 otherwise.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isascii#Test for 7-bit Ascii character
.pg
.hl 1 ^&Test for 7-bit Ascii character\&
.s 2
.c ;***********
.c ;* isascii *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isascii(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if c is a 7-bit Ascii character,
0 otherwise.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isatty##Test if terminal file
.pg
.hl 1 ^&Test if terminal file\&
.s 2
.c ;**********
.c ;* isatty *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
isatty(channel__number)
int		channel__number;
.s
.i -8;Description
.s
.fill
Return 1 if the file open on this channel is a
terminal-type device.
In general, this means the user's command terminal.
.s
Note that the argument is a file descriptor,
not a FILE * pointer.  File descriptors are used on
Unix to identify the channel on which the file is
open.  As Decus C does not support file descriptor
I/O processing, the logical unit number assigned
to a file by fopen() is used instead. Use isatty()
as follows:
.s
.nf
	if (isatty(fileno(fd))) ...
.s
.i -8;Bugs
.s
.fill
Note that Decus C channel numbers are not related to
Unix file descriptors.  Specifically,
.s
.nf
	isatty(0)
.s
.fill
does not necessarily check stdin.  Use
.s
.nf
	isatty(fileno(stdin))
.s
.fill
instead.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########iscntrl#Test for control character argument		;02
.pg
.hl 1 ^&Test for control character argument		;02\&
.s 2
.c ;***********
.c ;* iscntrl *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
iscntrl(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if C is a control character.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isdigit#Return TRUE if digit argument
.pg
.hl 1 ^&Return TRUE if digit argument\&
.s 2
.c ;***********
.c ;* isdigit *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isdigit(c);
int		c;
.s
.i -8;Description
.s
.fill
Return 1 if c is an Ascii digit, 0 otherwise.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isgraph#Test for graphic alphabetic argument
.pg
.hl 1 ^&Test for graphic alphabetic argument\&
.s 2
.c ;***********
.c ;* isgraph *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isgraph(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if c is a graphic character, 0 otherwise.
.s
Graphics characters include letters, digits, and punctuation.
Space, tab, etc. are not graphics.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########islower#Test for lower-case alphabetic argument
.pg
.hl 1 ^&Test for lower-case alphabetic argument\&
.s 2
.c ;***********
.c ;* islower *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
islower(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if C is a lower-case alphabetic character, 0 otherwise.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isprint#Return non-zero if printable argument
.pg
.hl 1 ^&Return non-zero if printable argument\&
.s 2
.c ;***********
.c ;* isprint *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isprint(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if c is a printable character, 0 otherwise.
.s
Printable characters include letters, digits, punctuation, and
the space character.  Tab, newline, etc. are not included.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ispunct#Return TRUE if punctuation argument
.pg
.hl 1 ^&Return TRUE if punctuation argument\&
.s 2
.c ;***********
.c ;* ispunct *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
ispunct(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if c is an Ascii punctuation, 0 otherwise.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isspace#Return non-zero if the character is "whitespace"
.pg
.hl 1 ^&Return non-zero if the character is "whitespace"\&
.s 2
.c ;***********
.c ;* isspace *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isspace(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if c is a whitespace character.  These consist of
space, backspace, newline, tab, and form-feed.  The carriage
return ('_\r') is also included.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isupper#Return TRUE if Upper-case alphabetic argument
.pg
.hl 1 ^&Return TRUE if Upper-case alphabetic argument\&
.s 2
.c ;***********
.c ;* isupper *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
isupper(c);
int		c;
.s
.i -8;Description
.s
.fill
Return 1 if c is an Upper-case alphabetic character, 0 otherwise.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########isxdigit########Return non-zero if hexadecimal digit
.pg
.hl 1 ^&Return non-zero if hexadecimal digit\&
.s 2
.c ;************
.c ;* isxdigit *
.c ;************
.s 2
.lm +8.nf
.i -8;Usage
.s
isxdigit(c);
int		c;
.s
.i -8;Description
.s
.fill
Return non-zero if C is a valid hexadecimal digit (0-9, A-F, a-f).
.s
.i -8;Bugs
.s
This routine is not present in other C libraries.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########itoa####Integer to Ascii
.pg
.hl 1 ^&Integer to Ascii\&
.s 2
.c ;********
.c ;* itoa *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
itoa(value, string)
int		value;
char		*string;
.s
.i -8;Description
.s
.fill
itoa converts the value to a (signed) decimal string.
The string is null-trailed.  itoa returns a pointer to
the trailing null.
.s
Note that the result can be computed (more flexibly)
by executing sprintf(buffer, "_%d", value);
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########itoa8###Convert integer to octal Ascii
.pg
.hl 1 ^&Convert integer to octal Ascii\&
.s 2
.c ;*********
.c ;* itoa8 *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
itoa8(value, buffer);
int		value;
char		*buffer;
.s
.i -8;Description:
.s
.fill
The value is converted to octal and stored in the buffer.
A pointer to the trailing null is returned.
.s
Note that the result can be computed (more flexibly)
by executing sprintf(buffer, "_%o", value);
.s
.i -8;Bugs:
.s
This routine does not generate leading zeros.  If they are
needed, use:
.s
.nf
	sprintf(buffer, "_%06o", value);
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########itoax###Convert an integer to hexidecimal Ascii
.pg
.hl 1 ^&Convert an integer to hexidecimal Ascii\&
.s 2
.c ;*********
.c ;* itoax *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
itoax(value, buffer);
int		value;
char		*buffer;
.s
.i -8;Description:
.s
.fill
Convert the integer to hexidecimal ascii.  The string
is null-trailed.  Values from 10 to 15 (decimal) are
represented by "A" to "F". Itoax() returns a pointer
to the trailing null.
.s
Note that the result can be computed (more flexibly)
by executing sprintf(buffer, "_%x", value);
.s
.i -8;Bugs:
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########kbin####Single Character Terminal Input
.pg
.hl 1 ^&Single Character Terminal Input\&
.s 2
.c ;********
.c ;* kbin *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
kbin();
.s
.i -8;Description
.s
.fill
Read one character from the console terminal.  The input character
is not echoed nor does the program delay until an
entire line has been entered.
.s
Note the following:
.s
This routine is very inefficient, as it requests operating-system
service for every character entered.
.s
Typing Control/C (or Control/Y on VMS) will cause the program
to exit immediately (as if the main() routine exited), closing
all files.
.s
On RT-11 (native and emulated), the operating system expands
<return> to a carriage return line feed sequence.  This will
be stripped by kbin() to just <return> ('_\r' in C).  Thus,
if the user program reads '_\n', a <line-feed> was typed.
Note that this is unlike the general C library.
.s
.i -8;Bugs
.s
On RSX, this routine requires the command terminal
having been assigned as logical unit number 1.
.s
In general, the input character will not have been echoed.
On RSTS/E and native RT11, the operating system may already
have echoed the character if you type the character before
the read request has been made.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########kbinr###Terminal Input Without Wait
.pg
.hl 1 ^&Terminal Input Without Wait\&
.s 2
.c ;*********
.c ;* kbinr *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
kbinr();
.s
.i -8;Description
.s
.fill
If a character has been typed on the console terminal,
return it (without echoing or waiting).  If no character
is available, kbin() returns -1 (EOF).
.s
Note the following:
.s
This routine is very inefficient, as it requests operating-system
service for every character entered.
.s
Typing Control/C (or Control/Y on VMS) will cause the program
to exit immediately.
.s
On RT-11, the operating system expands <return> to a carriage return
line feed sequence.  This will be stripped by kbin() to
just <return> ('_\r' in C).  Thus, if the user program reads
'_\n', a <line-feed> was typed.  Note that this is unlike the
general C library.
.s
.i -8;Bugs
.s
On RSX-11M, this routine depends on the fact that the console
terminal was assigned to LUN 1.
.s
On RSX-11M and VMS emulation mode, the typed character will not
be echoed.   On RSTS/E and RT11 modes, it may have already
been echoed by the operating system if the terminal user typed
before the program requests input.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########localtime#######Build time of day buffer
.pg
.hl 1 ^&Build time of day buffer\&
.s 2
.c ;*************
.c ;* localtime *
.c ;*************
.s 2
.lm +8.nf
.i -8;Usage
.s
struct tm *
localtime(tvec);
long		*tvec;	/* Time value pointer	*/
.s
struct tm {
  int	tm__sec;		/* Seconds		*/
  int	tm__min;		/* Minutes		*/
  int	tm__hour;	/* Hours		*/
  int	tm__mday;	/* Day of month		*/
  int	tm__mon;		/* Month 0 .. 11	*/
  int	tm__year;	/* Year - 1970		*/
  int	tm__wday;	/* Weekday, Sunday == 0	*/
  int	tm__yday;	/* Days since Jan 1	*/
  int	tm__isdst;	/* Daylight Saving if 1	*/
};
.s
.i -8;Description
.s
.fill
localtime() converts a Unix time value to a vector
of values.
.s
.i -8;Bugs
.s
Decus C doesn't correct for time zone or daylight
savings time.
.s
Leap year isn't done correctly.  The routine will fail
after Feb 28, 2100.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########malloc##Allocate and free memory
.pg
.hl 1 ^&Allocate and free memory\&
.s 2
.c ;**********
.c ;* malloc *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
malloc(size);		/* NULL if no space	*/
unsigned	size;	/* Number of bytes	*/
.s
mfree(p);
free(p);
char		*p;	/* Was allocated	*/
.s
.i -8;Description
.s
.fill
malloc() allocates the indicated number of bytes,
returning a pointer to the first.  If the allocation request
cannot be satisfied, malloc returns NULL.
The program image is expanded as needed.
.s
free() and mfree() return a buffer to free space.
Nothing is returned.
.s
See also the description of realloc() and sbrk().
.s
.i -8;Diagnostics
.s
The program may abort if free() is passed a random pointer
or if the program writes outside of an area reserved by malloc().
Note that only certain cases are trapped by this test and that
free will accept invalid addresses, which will cause mysterious
failures.  The isalloc() function may be used to test whether
a block of memory has been allocated by malloc().
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########maxmin##Maximum and Minimum Routines
.pg
.hl 1 ^&Maximum and Minimum Routines\&
.s 2
.c ;**********
.c ;* maxmin *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
max(a,b)
int		a;
int		b;
.s
min(a,b)
int		a;
int		b;
.s
unsigned
maxu(a,b)
unsigned	a;
unsigned	b;
.s
unsigned
minu(a,b)
unsigned	a;
unsigned	b;
.s
.i -8;Description
.s
.fill
max() and min() return, respectively, the maximum and minimum of their
two arguments, considered as signed integers.  maxu() and minu() are
the same but consider their arguments to be unsigned.
.s
Note:  If you are interested in getting the fastest code possible and
have some knowledge of the relative sizes of a and b, arrange for a to
be chosen most often.  (It's unlikely the difference will be noticable
in virtually all cases.)
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########memdmp##Dump memory or registers
.pg
.hl 1 ^&Dump memory or registers\&
.s 2
.c ;**********
.c ;* memdmp *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
memdmp(start, end);	/* Memory dump routine	*/
char		*start;	/* First to dump	*/
char		*end;	/* Last to dump		*/
.s
regdmp();		/* Register dump	*/
.s
.i -8;Description
.s
.fill
Dump registers and/or memory.  All registers are preserved.
.s
If memdmp is called with a zero argument, the stack will
be dumped.
.s
Memdmp and regdmp are independent of the C I/O library.
However, on RSX, they assume that the console terminal
has been assigned as LUN 1.
.s
.i -8;Bugs
.s
Condition codes are not preserved.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########msg#####Print a message on the command terminal
.pg
.hl 1 ^&Print a message on the command terminal\&
.s 2
.c ;*******
.c ;* msg *
.c ;*******
.s 2
.lm +8.nf
.i -8;Usage
.s
msg(text)
char		*text;
.s
.i -8;Description
.s
.fill
Print a message on the console terminal.  This routine
does not use the standard library.  Return: all registers
are preserved.
.s
.i -8;Bugs
.s
On RSX, this routine requires the command terminal
having been assigned as logical unit number 1.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########msize###Get Size of a Memory Block
.pg
.hl 1 ^&Get Size of a Memory Block\&
.s 2
.c ;*********
.c ;* msize *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
unsigned int
msize(buff)
char *buff;
.s
.i -8;Description
.s
.fill
msize() returns the size of a block of memory allocated by calloc(),
malloc(), or realloc().  If buff did not, in fact, come from one of
these routines, the program will crash about 75_% of the time; the
other 25_% , nonsense will be returned.  (This behavior is due to a
simple-minded validation algorithm.)  However, buff == NULL is valid
and will return 0.
.s
Note that the value returned is the amount of memory that malloc()
(or whoever) actually allocated, which may not be exactly the same
as the program asked for.  (Currently, all allocations are rounded
up to an even number of bytes.)
.s
.i -8;Bugs
.s
Not portable.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########peek####Peek at a location in RSTS/E
.pg
.hl 1 ^&Peek at a location in RSTS/E\&
.s 2
.c ;********
.c ;* peek *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
peek(location)
int		*location;
.s
.i -8;Description
.s
.fill
Peek to a location in the RSTS/E monitor.
.s
The job will be aborted with a BPT if not running under
RSTS/E.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########perror##Print Library Error Message
.pg
.hl 1 ^&Print Library Error Message\&
.s 2
.c ;**********
.c ;* perror *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
perror(text)
char		*text;
.s
.i -8;Description
.s
.fill
An error message is written to stderr, using the
current i/o library error (stored in $$ferr).
Text is prepended to the message.
.s
.i -8;Diagnostics
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########printf##Formatted print routine
.pg
.hl 1 ^&Formatted print routine\&
.s 2
.c ;**********
.c ;* printf *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
printf(format, arg1, ...)
char		*format;
.s
fprintf(iov, format, arg1, ...)
FILE		*iov;
char		*format;
.s
char *
sprintf(buffer, format, arg1, ...);
char		*buffer;
char		*format;
.s
$$prnt(format, argvec, iov)
char		*format;
int		*argvec[];
FILE		*iov;
.s
.i -8;Description
.s
.fill
printf() converts, formats, and prints its arguments, under control
of the first argument, writing output via putchar().  fprintf()
writes its output to the indicated file.  sprintf() writes its
output to the indicated string buffer.  $$prnt() is the internal
print formatter which is called by printf, etc.
.s
sprintf() returns a pointer to the EOS at the end of the output
buffer.  This is not necessarily transportable.
.s
The format argument is a character string which contains two
types of objects: plain characters, which are simply copied
to the output stream, and conversion specifications, each of
which causes conversion and printing of the next successive
argument to printf.
.s
Each conversion specification is introduced by the character _%.
Following the _%, there may be
.s
.nf
 - an  optional  minus  sign  "-"  which  specifies left
   adjustment of the converted argument in the indicated
   field.
.s
 - an optional digit string specifying field  width;  if
   the  converted argument has fewer characters than the
   field width, it will be blank-padded on the left  (or
   right, if  the  left-adjustment  indicator  has  been
   given) to make up the  field  width.   If  the  field
   width  is  specified as '*' or '?', the next argument
   is used.  (Note: '?' is  obsolete.)  If the  width is
   specified  with a leading  zero,  zeros are  used for
   padding instead of blanks.  This zero  does NOT imply
   an octal field width.  For example, assume the  value
   123 is to be printed:
	_%d	"123"
	_%5d	"  123"
	_%-5d	"123  "
	_%05d	"00123"
.s
 - an  optional  period "." which serves to separate the
   field width from the next digit string;
.s
 - an optional digit string (precision) which  specifies
   the number of digits  to  appear  after  the  decimal
   point for e- and f-conversion, or the maximum  number
   of  characters  to be printed from a string.   If the
   precision is  specified  as  '*'  or  '?',  the  next
   argument is used.
.s
 - a character which indicates the  type  of  conversion
   to be applied.
.s
.fill
The conversion characters and their meanings are
.s
.nf
 d	Signed-decimal
 u	Unsigned-decimal
 o	Octal
 X	Hexadecimal, 10-15 are represented by A-F
 x	Hexadecimal, 10-15 are represented by a-f
.s
	The  integer  argument  is converted to decimal,
	octal, or hexadecimal   notation   respectively.
	Any   of   the   conversion  characters  may  be
	preceeded  by  'l'  to  signal  "long"   integer
	argument.   Note  that the Unix usage of capital
	letters  to  represent  long  arguments  is  not
	supported.
.s
			     Note
.s
	    In order to  eliminate  the  very  large
	    floating  point conversion routines from
	    most programs, a program which  requires
	    floating  point  conversion must request
	    the  double  to ascii conversion routine
	    explicitly.    The  following  sequences
	    show how this is done:
.s
        RSX: TKB task=source,LB:DTOA,LB:C/LB
       RT11: LINK source,C:DTOA,C:SUPORT,C:CLIB/BOT:2000
.s
	    If this is not done, any use of  the  _%f
	    _%e, or _%g conversions will result in  an
	    error  message.   On RSTS/E, the library
	    build procedure creates  RTDTOA.OBJ  and
	    RXDTOA.OBJ in the library account.
.s
	    ** Floating point support has not yet **
	    ** been fully implemented.  The float **
	    ** or  double  value must be the last **
	    ** or only argument to printf.   Only **
	    ** single-precision   conversion   is **
	    ** presently available.		  **
.s
 f	The  argument  is  converted to decimal notation
	in the style "[-]ddd.dd" where the number of d's
	after  the  decimal  point  equals the precision
	specification for the argument.  If precision is
	missing, 6 digits are given; if explicitly 0, no
	digits and no decimal point  are  printed.   The
	argument  should  be  float or double.  "_%lf" or
	"_%F" must be used to signal "long (i.e.  double)
	argument."  This is a restriction of Decus C.
.s
 e	The float or double argument is converted in the
	style "[-]d.ddde+-dd" where there is  one  digit
	before the decimal point and the number after is
	equal  to  the  precision   specified   for  the
	argument;  if the precision is missing, 6 digits
	are produced.  "_%le" or "_%E"  must  be  used  to
	signal "long (i.e. double) argument."  This is a
	restriction of Decus C.
.s
 g	Floating point:  "_%f" format if  suitable,  else
	"_%e"  format.   "_%lg"  or  "_%G"  must be used to
	signal "long (i.e. double) argument."  This is a
	restriction of Decus C.
.s
 c	The argument character is printed.   (Note  that
	'lc' takes a long integer argument.)
.s
 r	Remote format.  The next  printf()  argument  is
	the format.  Note that this is not a subroutine.
	The current format is not processed further. For
	example:
.s
		bug(args)
		{
			error("Error at _%r", _&args);
		}
.s
	This routine might be called as follows:
.s
		bug("Error _%d at _%s_\n", val, name);
.s
	_%r is not transportable to  all  implementations
	of  the  standard  library.  It does not word on
	Vax-11 C, for example.  $$prnt may  be  used
	as shown below for similar functionality.
.s
 s	The  argument is taken to be a string (character
	pointer) and  characters  from  the  string  are
	printed  until  a  null  character  or until the
	number of characters indicated by the  precision
	specification  is   reached;   however   if  the
	precision   specification  is  0  or missing all
	characters up to null are printed.
.s
.fill
If no recognizable character appears after the _%, that character
is printed; thus _% may be printed by the use of the string _%_%.
In no case does a non-existant or small field width cause truncation
of a field; padding takes place only if the specified field
width exceeds the actual width.  Characters generated by printf()
are printed by calling putchar().
.s
$$prnt() is the internal print formatter called by all
"top-level" print functions.  It is functionally identical
to the Unix and Vax-11 C __doprnt() library routine.  Unfortunately,
the leading '__' conflicts with RSX-11M file services library
routine conventions, requiring the use of the Decus C unique
"$$" prefix. If your programs wish to call $$prnt,
a potentially transportable procedure would be:
.s
.nf
    _#ifdef decus
	$$prnt(format, args, iov);
    _#else
	__doprnt(format, args, iov);
    _#endif
.s
.fill
You should assume, however, that __doprnt() is not necessarily
present on all implementations of the "standard library."
.s
.i -8;Bugs
.s
e, f, and g conversion only work for single-precision floating
point.  Use of "_%lf" etc. is not transportable and may change.
It may also cause problems in the C compiler (as the standard
says that subroutine calls take double (not short floating)
arguments.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########profile#Print profile data
.pg
.hl 1 ^&Print profile data\&
.s 2
.c ;***********
.c ;* profile *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
$$prof();
.s
extern	int	$$prnl;
.s
extern	char	*$$pfil;
.s
.s
.i -8;Description
.s
.fill
$$prof is called by the run-time library when the program
exits if any functions have been executed that were compiled
with the profile option.
.s
A profile file is written to file "profil.out".  This is defined
by a global symbol $$pfil.  By setting $$prnl to NULL, the
profile file is supressed.
.s
By changing the value of $$prnl, the program can control the
number of profile entries written on each line.  The
default value writes 4 entries per line.  Changing
$$prnl to N writes N entries on each line, simplifing
post-processing (sorting) of the data.  For example:
.s
.nf
	$$prnl = 0;
.s
.fill
writes each entry on a separate line.
.s
The following example shows how to control the output file
name:
.s
.nf
	extern char *$$pfil;
	...
	$$pfil = "ti:";	/* Output to ti:	*/
	$$pfil = NULL;	/* No profile output	*/
.s
.s
.fill
Data is written using the format string "_%8s _%6u".
If more than one entry is written to a line, succeeding
entries will be preceeded by one space.
.s
Note that, by writing the data one entry per line, the
profile output may easily be sorted either by function name
or by frequency count.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########putc####Output one character to a file
.pg
.hl 1 ^&Output one character to a file\&
.s 2
.c ;********
.c ;* putc *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
.s
putchar(c);
char		c;
.s
putc(c, iop);
char		c;
FILE		*iop;
.s
.i -8;Description
.s
.fill
Putchar() writes one character to the standard output file.
Putc() writes one character to the named output file.
.s
Normally, the character is returned.  EOF is returned on
errors.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########puts####Output a string to a file
.pg
.hl 1 ^&Output a string to a file\&
.s 2
.c ;********
.c ;* puts *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
puts(s);
char		*s;
.s
fputs(s, iop);
char		*s;
FILE		*iop;
.s
fputss(s, iop);
char		*s;
FILE		*iop;
.s
.i -8;Description
.s
.fill
puts() writes a string to the standard output.  It appends
a newline after the string.
.s
fputs() writes a string to the indicated file.  No newline
is appended.
.s
fputss() writes a string to the indicated file.  A newline
is appended.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########putw####Output a binary integer to a file
.pg
.hl 1 ^&Output a binary integer to a file\&
.s 2
.c ;********
.c ;* putw *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
putw(word, iop);
int		word;
FILE		*iop;
.s
.i -8;Description
.s
.fill
putw() writes one word to the indicated file.
It returns EOF on error, 0 on success
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########qset####Add memory to the RT11 queue area
.pg
.hl 1 ^&Add memory to the RT11 queue area\&
.s 2
.c ;********
.c ;* qset *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
qset(number)
int	number;	/* How many elements		*/
.s
.i -8;Description
.s
.fill
Allocate sufficient memory for the requested number of
queue elements.  If sufficient memory cannot be allocated,
do nothing.
.s
.i -8;Bugs
.s
This routine is ignored on RSX.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########r50toa##Convert Radix-50 to Ascii
.pg
.hl 1 ^&Convert Radix-50 to Ascii\&
.s 2
.c ;**********
.c ;* r50toa *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
r50toa(buff, r5vec, r5cnt);
char		*buff;	/* Output text buffer	*/
int		*r5vec;	/* Input rad50 buffer	*/
int		r5cnt;	/* How many rad50 words	*/
.s
.i -8;Description:
.s
.fill
Convert r5cnt words of radix 50 data to Ascii.  All letters
will be in upper-case.  The output buffer will not be
null-trailed, nor will blank fields be supressed.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rand####Random number generator
.pg
.hl 1 ^&Random number generator\&
.s 2
.c ;********
.c ;* rand *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
long
rand()
.s
extern long	seed;	/* Random number seed	*/
.s
.i -8;Description
.s
.fill
Generate a pseudorandom number.
.s
The algorithm is:
.s
.nf
	seed = (69069 * seed + 1);
	temp = (seed >> 8) _& 32767;
	return ((max == 0) ? temp : temp _% max);
.s
.fill
The algorithm is based on the mth$random function in the
VMS common run-time library.  As rand() returns a 16-bit
integer, the middle of the generated random number is used.
.s
Note that the algorithm is prone to nonrandom sequences when
considering the next pseudorandom number.
.s
.i -8;Bugs
.s
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########realloc#Reallocate memory
.pg
.hl 1 ^&Reallocate memory\&
.s 2
.c ;***********
.c ;* realloc *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
realloc(buff, size);
char		*buff;	/* Buffer from malloc()	*/
unsigned	size;	/* New size for buff	*/
.s
.i -8;Description
.s
.fill
Realloc() changes the size of the block pointed to by buff,
returning a pointer to the (possibly moved) block.  The
contents will be unchanged up to the lesser of the
new and old sizes.
.s
Realloc() also works if buff points to a block freed since the
last call of malloc(), realloc(), or calloc();  thus sequences
of free(), malloc(), and realloc() can exploit the search
strategy of malloc() to do storage compaction.
.s
See also the description of malloc(), calloc() and sbreak().
.s
The following program segment illustrates use of realloc()
to expand and compact storage.
.s
.nf
	_#ifndef	vms
	static char *compact__work;
	_#endif
.s
	main() {
		/*
		 * Initialize work element
		 * for compact()
		 */
	_#ifndef	vms
		compact__work = malloc(1);
	_#endif
		...
	}
.s
	char *
	compact(old__ptr, new__size)
	char	*old__ptr;
	int	new__size;
	/*
	 * Reallocate the area allocated to old__ptr,
	 * originally allocated by a call to malloc(),
	 * so that it will contain new__size bytes (which
	 * may be greater or less than the current size).
	 * Return a pointer to the new area.
	 * Note:  error handling is not shown.
	 */
	{
		extern char *realloc();
.s
		free(old__ptr);
	_#ifndef	vms
		free(compact__work);
		compact__work = malloc(1);
	_#endif
		return(realloc(old__ptr, new__size));
	}
.s
.fill
compact__work must not be used on the native vax compiler
(Vax-11 C) as the internal free memory buffers are organized
in a different and incompatible manner.
.s
.i -8;Diagnostics
.s
The program will abort if buff is not the address of a buffer
allocated by malloc() or calloc().
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rewind##Rewind a file for re-reading
.pg
.hl 1 ^&Rewind a file for re-reading\&
.s 2
.c ;**********
.c ;* rewind *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
rewind(iop);
FILE		*iop;
.s
.i -8;Description
.s
.fill
Rewind the indicated file.  Return -1 if failure, 0 if ok.
.s
On RT11, if the file was opened for output, it is closed and
reopened for input.
.s
.i -8;Diagnostics
.s
RT11 exits fatally if the file name won't parse.  As the
filename was stored by fopen(), this probably means that
the user program has stored randomly in memory.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rindex##Find Last Instance of a Character in a String
.pg
.hl 1 ^&Find Last Instance of a Character in a String\&
.s 2
.c ;**********
.c ;* rindex *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
rindex(stng, chr)
char		*stng;	/* String to search in	*/
char		chr;	/* Byte to search for	*/
.s
.i -8;Description
.s
.fill
If chr is in stng, return a pointer to the last instance it.
If not, return NULL.
.s
.i -8;Bugs
.s
Obsolete, use strrchr() instead.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rstsys##Execute a RSTS EMT
.pg
.hl 1 ^&Execute a RSTS EMT\&
.s 2
.c ;**********
.c ;* rstsys *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
rstsys(emt)
.s
.i -8;Description
.s
.fill
Execute a rsts emt, returning the error code.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rtemt###Execute a RT11 EMT
.pg
.hl 1 ^&Execute a RT11 EMT\&
.s 2
.c ;*********
.c ;* rtemt *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
rtemt(emt, r0value)
.s
.i -8;Description
.s
.fill
Execute a rt11 emt after loading r0.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########salloc##Allocate local memory
.pg
.hl 1 ^&Allocate local memory\&
.s 2
.c ;**********
.c ;* salloc *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
salloc(n);		/*   NULL if no space	*/
.s
.i -8;Description
.s
.fill
Allocate the requested number of bytes on the run-time stack,
returning a pointer to the first byte allocated.  The storage
will be reclaimed automatically when the function that called
salloc() exits.
.s
Note that storage so allocated cannot be explicitly freed.
.s
salloc() returns NULL if allocating the requested space would
cause the run-time stack to go below 1000 octal.
.s
Note: it is essential that salloc() be called with a "clean
stack".  I.e, the program must not execute
.s
.nf
	subr(salloc(10), 20);
.s
.fill
Instead, it should execute
.s
.nf
	temp = salloc(10);
	subr(temp, 20);
.s
.i -8;Diagnostics
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########sbrk####Allocate memory from operating system
.pg
.hl 1 ^&Allocate memory from operating system\&
.s 2
.c ;********
.c ;* sbrk *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
sbrk(amount)
int		amount;
.s
.i -8;Description
.s
.fill
sbrk() allocates the indicated amount of memory from
the operating system.  The allocation is permanent.
.s
.i -8;Diagnostics
.s
NULL is returned if the memory requested is not available.
There is no "shortened" return.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########scanf###Formatted input conversion
.pg
.hl 1 ^&Formatted input conversion\&
.s 2
.c ;*********
.c ;* scanf *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
scanf(fmt, pointer(s))
char		*fmt;	/* Format string	*/
.s
fscanf(fd, fmt, pointer(s))
FILE		*fd;	/* Input file pointer	*/
char		*fmt;	/* Format string	*/
.s
sscanf(buf, fmt, pointer(s))
char		*buf;	/* Input text buffer	*/
char		*fmt;	/* Format string	*/
.s
$$scan(fmt, argp, iov)
char		*fmt;	/* Format string	*/
int		*ptr[];	/* Pointer vector	*/
FILE		*iov;	/* Input file des.	*/
.s
.i -8;Description
.s
.fill
Using the format string, these functions parse the input file
(or given text file), storing the results in the pointer
arguments.  The three user-callable routines differ as follows:
.s
.nf
	scanf	Reads from the standard input file.
.s
	fscanf	Reads from the indicated file.
.s
	sscanf	Reads from the text buffer.
.s
.fill
$$scan() is an internal routine called by the above to actually
parse the input text.  It is functionally identical to the Unix
and Vax-11 C __doscan() routine.  Unfortunately, the leading
'__' conflicts with RSX file control service routine naming
conventions.
.s
The format string may contain control characters to direct
the conversion of the input textual data:
.s
.nf
  ' '	Blanks, tabs, or newlines are ignored in  format
	strings.  To match whitespace, use "_%[ _\t_\n]".
.s
   x	An  ordinary  character  (not  _%) must match the
	next input character.
.s
   _%	Conversion specifications consist of a  '_%',  an
	optional   '*'  (to   supress   assignment),  an
	optional  maximum  numeric  field  width,  and a
	conversion specifier.
.s
.fill
Unless value assignment was supressed by the '*' format modifier,
the next field is converted and stored in the variable pointed to
by the corresponding argument.  An input field is defined as a
string of non-space characters, extending to the first inappropriate
character or until the field width (if given) is exhausted.
.s
The following conversion characters are specified:
.s
.nf
  _%	A single  '_%'  is  expected  in  the  input,  no
	assignment is done.
.s
  d	An integer  of  the  specified  class  (decimal,
  o	octal,  or  hexadecimal)   is   expected.    The
  x	corresponding  argument  should  be  an  integer
	pointer.   If   the  format specifer is given in
	upper-case, or preceeded by 'l', a long  integer
	will be stored.  For example, "_%ld" is identical
	to "_%D".   Leading  whitespace  will be ignored.
	A null field is not converted.  Scanf()  returns
	a  count  of the number of fields converted, but
	does not indicate which fields were ignored.
.s
  s	A character string is expected.  The input field
	will be terminated by a space, tab, or  newline.
	The corresponding argument  should  point  to  a
	buffer large enough to contain the  text  and  a
	terminating  NULL.   Leading  whitespace will be
	ignored.
.s
  c	A single character is read. Leading white  space
	is not supressed  --  to read the next non-blank
	character, use "_%1s".  If a field width is given
	the  corresponding  argument  is  a pointer to a
	vector of characters and the indicated number of
	characters are read.
.s
  e	A floating-point number is converted and  stored
  f	appropriately.    If  the  format  indicator  is
  g	capitalized, or  preceeded  by  'l',  a  double-
	precision  floating-point number will be stored.
	The floating-point format is the same as in  the
	C language:   an  optionally  signed  string  of
	digits  possibly  containing  a  decimal  point,
	followed by an  optional  exponent  field  which
	consists  of  an  'E'  or  'e'  followed  by  an
	optionally signed integer.
.s
 [ ]	A string not  to  be  delimited  by  white-space
	characters.  Unless the first character is  '_^',
	the format constitutes an acceptance  list:  the
	input field is all characters  until  the  first
	character  which  is  not  in  the  set   within
	brackets.   Note  that leading whitespace is not
	ignored.
.s
	If the first character after the left bracket is
	'_^', the format defines a stop list:  the  input
	field   is   all   characters  until  the  first
	character specified within the bracketed string.
.s
	The corresponding argument points to a character
	vector.  The result will be null-terminated.
.s
.fill
Scanf() returns the number of successfully matched and
assigned input items.  If the end of input was reached,
EOF (-1) is returned.  For example:
.s
.nf
	main()
	{
		register int	c;
		int		i;
		char		text[10];
.s
		c = scanf("_%d_%9s", _&i, _&text);
		printf("i = _%d, text = _%s_\n",
			i, text);
	}
.s
.fill
If the input line is "150   foobar", the program will print:
.s
.nf
	i = 150, text = foobar
.s
.i -8;Diagnostics
.s
.fill
Scanf() returns -1 if an end of file (end of string) condition
exists and no data was stored.  It returns -2 if a palpably
incorrect format, such as "_%" is encountered.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########screen##Screen I/O Primitives
.pg
.hl 1 ^&Screen I/O Primitives\&
.s 2
.c ;**********
.c ;* screen *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
scdown()		/* Roll screen down	*/
.s
scerln(line, column)	/* Erase to end of line	*/
int		line;	/* Line number		*/
int		column;	/* Column number	*/
.s
scerpg(line, column)	/* Erase to end of page	*/
int		line;	/* Line number		*/
int		column;	/* Column number	*/
.s
.s
char *
scget(buff, size, text)
			/* Prompt and read	*/
char		*buff;	/* Returned text	*/
int		size;	/* Buffer size		*/
char		*text;	/* Optional prompt	*/
.s
.s
int
sclmargin(lmarg)	/* Set left margin	*/
int		lmarg;	/* New left margin	*/
.s
.s
scout(line, column, text)
			/* Write text to screen	*/
int		line;	/* Line number		*/
int		column;	/* Column number	*/
char		*text;	/* Text to write	*/
.s
.s
scput(oldbf)		/* Reset buffer mode	*/
char		*oldbf;	/* Previous buffer	*/
.s
.s
int
scset(newbf, size, old)
			/* Set buffer mode	*/
char		*newbf;	/* New buffer		*/
char		size;	/* Size of new buffer	*/
char		**old;	/* Previous buffer save	*/
.s
.s
int
scsettype(type)		/* Force terminal type	*/
int		type;	/* Terminal type	*/
.s
.s
int
sctype()		/* Return terminal type	*/
.s
.s
.i -8;Description
.s
.fill
Based on the "terminal independent screen procedures"
in the VAX/VMS runtime library, these routines provide
an efficient, operating-system independent, interface
to (DIGITAL) video terminals.  They work correctly with
VT52 and VT100 (ANSI) terminals and do nothing disasterous
if the terminal is not a video terminal.
.s
Note that they only work with the user's console (stderr)
terminal.
.s
The following definitions should be noted:
.s
.nf
	line	The line on  the  screen  to  which  the
		cursor  is  to be moved.  The top of the
		screen is line one.   If  line  is  less
		than  or  equal to zero, the cursor does
		not move.
.s
	column	The column to which the cursor is to  be
		moved.  This value will be offset by the
		left-margin value (if one is set).   The
		default  left-hand margin is column one.
		If column  (after  offsetting)  is  less
		than  or  equal to zero, the cursor does
		not move.
.s
	text	A null-terminated string to  be  output.
		If  text  equals NULL, nothing is output
		and internal buffers are flushed.
.s
.fill
The routines are designed to operate in a buffered mode,
allowing the program to format an entire screen of information
and present this data with one call to the monitor-level
I/O service.  This is particularily efficient on multi-user
or networked systems.  The routines scput() and scset()
establish and release buffer mode.  Buffers can be chained
together, allowing a subroutine to establish a buffer
independently of its callers.  Each buffer must be at
least 12 bytes long.  Longer buffers are more efficient.
.s
Although multiple buffers can be established, only one is
active at any particular time.  If a call to scset() establishes
a new buffer, the contents of the previous buffer are copied
to the new buffer and the previous buffer is set empty.
If scput() releases a buffer, its contents are copied to
any higher-level buffer.
.s
The first call to any routine determines the terminal type.
Note that on native RT11, if the screen handler decides that the
terminal may be a scope, it will send an "identify" request
"<ESC>/Z" to determine whether it is a VT52 or VT100.  If the
screen routine decides that the terminal is not a scope, cursor
move operations are ignored and text is output to "stderr".
If the screen handler decides that the terminal is a VT100,
the "Enter ANSI mode" escape sequence will be sent to the terminal.
Similarily, if the terminal is thought to be a VT52, the
"Enter VT52 emulation mode" sequence will be sent.  The screen
handler will not record the current mode for subsequent resetting.
.s
Screen output will be to the "stderr" device (explicitly the
console terminal).  Input will be from the "stdin" device. If a
local buffer has been established, the screen handler calls
operating system service directly, bypassing the "standard" I/O
library. This means that output formatting should use sprintf() to
format a user buffer which is subsequently output using scout().
Furthermore, the newline '_\n' character will not be expanded
to a "carriage return/line feed" sequence.  It is thus highly
recommended that the user program perform all formatting by passing
appropriate row and column values to scout().
.s
The following routines are defined:
.s
.s
.nf
	scdown()
.s
.fill
The cursor is moved up one line on the screen.  If the
cursor was already at the top line on the screen, all
lines are moved down one line, the top line is replaced
with a blank, and the data that was on the bottom line
is lost.  (VMS Lib$Down__Scroll)
.s
.s
.nf
	scerln(line, column)
.s
.fill
Erase the screen from the indicated (or current) cursor
position to the end of the current line.  (VMS Lib$Erase__Line)
.s
.s
.nf
	scerpg(line, column)
.s
.fill
Erase the screen from the indicated (or current) cursor
position to the end of the screen. (VMS Lib$Erase__Page)
.s
.s
.nf
	scget(buff, size, text)
.s
.fill
Output the prompt text, if given, then flush any buffered text.
Read text input by the terminal user (from stdin) to the buffer.
Scget() returns NULL on end of file or error. (Scget calls
fgetss() to perform the read.)  (VMS Lib$Get__Screen)
Programs using scget() to read from the keyboard should not
use the input redirection capabilities of the C library.
.s
.s
.nf
	sclmargin(lmarg)
.s
.fill
Define a new left margin (if lmarg is greater than zero).
Sclmargin() returns the old lmarg value.  After calling
sclmargin(), an output request to column one will be written
to column lmarg.
.s
.s
.nf
	scout(line, column, text)
.s
.fill
Move the cursor to the indicated line and column and output
the text.  If line or column are less than or equal to zero,
the cursor is not moved.  If text is equal to NULL, the
internal buffers are flushed (possibly after cursor movement).
(VMS Lib$Put__Screen)
.s
.s
.nf
	scput(oldbf)
.s
.fill
Scput() terminates the current buffering mode and reverts to
the previous mode as specified in the parameter.  If oldbuf
is NULL, buffering is terminated and the current buffer is
flushed to the screen.  If oldbuf is not zero, it is taken
as the location of a previous buffer (established by calling
scset()) and the current buffer is copied into the previous
buffer.  The previous buffer is then made current.
(VMS Lib$Put__Buffer)
.s
.s
.nf
	scset(newbf, size, old)
.s
.fill
Scset() establishes a new buffer for the screen routines.
Newbf is a character array whose size is given in parameter
size (Size must be at least 12 bytes.)  Old is the address
of a word which will be set to the previous buffer (if any).
This is used for a subsequent call to scput().  Scset returns
zero if correct, returning -1 if size was less than 12 bytes.
(VMS Lib$Set__Buffer).
.s
A larger buffer will generally yield a more efficient program
as the number of operating system requests will be minimized.
For example, for a typeout program, a buffer size of 800 bytes
might well be used.
.s
.s
.nf
	scsettype(type)
.s
.fill
This routine sets the terminal type, overriding the definition
supplied by the operating system.  scsettype() returns what it
thought the type was (as if sctype were called).  The type
must be one of the following values:
.s
.nf
	0	Not a video terminal
	65	VT52 or VT100 in VT52 mode
	97	VT100 in ANSI mode
.s
.fill
If type has some other value, scsettype() does not change the
type.
.s
.s
.nf
	sctype()
.s
.fill
This routine returns the terminal type.  The following types
are defined:
.s
.nf
	0	Not a video terminal
	1	Unknown video terminal
	1+64	VT52
	1+96	VT100
.s
.fill
Note: On intitialization, a VT100 will be forced into
VT52 or VT100 mode depending on the determined type.
.s
.s
Note the following normal use of scset() and scput()
.s
.nf
	char	**oldbuf;
	char	mybuf[100];
.s
	main()
	{
	    /*
	     * Setup
	     */
	    scset(mybuf, sizeof mybuf, _&oldbuf);
	    /*
	     * Usage
	     */
	    scout(1, 1, "Stuff");
	    /*
	     * Termination (force out last buffer)
	     */
	    scout(0, 0, NULL);
	    scput(oldbuf);
	}
.s
.s
.i -8;Bugs
.s
.fill
These routines are fairly inefficient if no buffer is supplied.
.s
Determining the terminal type turns out to be a somewhat painful
process which is different for the various operating systems:
.s
On native RSX-11M and VMS compatibility mode, the QIO
"get multiple characteristics function" is executed.
If the terminal is a scope and can generate escape
sequences, it is assumed to be a VT100, otherwise, if
it is a scope, it is assumed to be a VT52.
.s
On all RSTS/E systems, the executive "get terminal characteristics"
request is executed, the determination proceeding as above.
.s
On native RT11 systems, a peek sequence described in the
RT11 Software Support manual is used to determine whether
the terminal is a scope.  (This may not work on the XM monitor.)
If so, an "<ESC>/Z" is sent to request identification and various
tests are made on the returned text.  Note that, if the terminal
does not reply to the inquiry, the program will hang.
.s
On RSX-11M systems (even emulated), if a local buffer is
established by calling scset(), output will be done using
the IO.WAL QIO function.
.s
.s
While based on the VMS Run-time library screen I/O primitives,
there are a few differences that may be of interest:
.s
The VMS library standard assumes that subroutines generally
return a status code, permit missing parameters, and return
error codes on being presented with invalid arguments.  The
routines described here do nothing when arguments are out of
range.  Also, there are no optional arguments (optionality
is provided for by specifing zero or NULL argument values).
Where appropriate, arguments have been ordered "line__no, col__no,
other".
.s
A more significant difference is that the VMS library writes
to the standard output device (SYS$OUTPUT: or TT:) while
these routines write to the user's terminal (SYS$ERROR or CO:).
scget() reads from the standard input.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########setcc###Trap Control/C (RSTS/E RT11 mode only)
.pg
.hl 1 ^&Trap Control/C (RSTS/E RT11 mode only)\&
.s 2
.c ;*********
.c ;* setcc *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
setcc(function);
extern int function();
.s
.i -8;Description
.s
.fill
The function is defined as a CTRL/C trap routine.  Executing
setcc(0) disables CTRL/C trapping.
.s
Note: If the routine is reentered (because CTRL/C was
typed during the execution of function()), the program aborts.
.s
.i -8;Bugs
.s
Runs on RSTS/E RT11 mode only.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########setjmp##Execute non-local goto
.pg
.hl 1 ^&Execute non-local goto\&
.s 2
.c ;**********
.c ;* setjmp *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
_#include <setjmp.h>
.s
setjmp(env);
jmp__buf		env;	/* Static save buffer	*/
.s
longjmp(env, val)
jmp__buf		env;	/* Was set by setjmp	*/
int		val;	/* Value to return	*/
.s
.i -8;Description:
.s
.fill
These routines are useful for dealing with errors and interrupts
encountered in a low-level subroutine of a program.
.s
Setjmp() saves its stack environment in env for later
use by unwind().  It returns 0.
.s
Longjmp(env) restores the environment saved by the last call of
setjmp(env). It then returns in such a way that execution continues
as if the call of setjmp() had just returned.  All accessible data
have values as of the time setjmp() was called.  The return from
setexit() will have the specified value.  For example:
.s
.nf
    _#include <setjmp.h>
    jmp__buf	env;
    ...
    main() {
	...
	if (setjmp(_&env) != 0) {
	    err__exit();	/* Longjmp called	*/
	}
	else {
	    process();	/* Setjmp  called	*/
	}
	...
    process() {
	...
	longjmp(_&env);
.s
.fill
The routine that called setjmp() must still be active when longjmp()
is called.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########sleep###Delay job a given number of seconds
.pg
.hl 1 ^&Delay job a given number of seconds\&
.s 2
.c ;*********
.c ;* sleep *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
sleep(delay);
int		delay;
.s
.i -8;Description
.s
.fill
Sleep a specified number of seconds.
.s
.i -8;Bugs
.s
On RSTS/E, the reader should refer to the description
of SLEEP() in the operating system documentation.
.s
On native RSX-11M or RSX emulated by VMS, sleep() executes
a mark-time (MRKT$) and wait sequence, using event flag 1.
.s
On native RT11, sleep() executes a .TWAIT executive request.
.s
Sleep flushes stdout and stderr.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########sprintf#Formatted Numeric Conversion
.pg
.hl 1 ^&Formatted Numeric Conversion\&
.s 2
.c ;***********
.c ;* sprintf *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
sprintf(buffer, format, arg1, ...);
char		*buffer;
char		*format;
.s
.i -8;Description
.s
.fill
sprintf() converts and formats its arguments, writing the
result to the indicated string buffer.
.s
For information on formatting, please refer to the description
of printf.
.s
sprintf() returns a pointer to the EOS byte at the end
of the output buffer.  Note, however, that this feature
is not transportable to other C implementations.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########sscanf##Formatted input conversion
.pg
.hl 1 ^&Formatted input conversion\&
.s 2
.c ;**********
.c ;* sscanf *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
sscanf(buf, fmt, pointer(s))
char		*buf;	/* Output buffer	*/
char		*fmt;	/* Format string	*/
.s
.i -8;Description
.s
.fill
sscanf() parses the input string, in buf, according to the indicated
format descriptor, storing the results in the pointer arguments.
It returns the number of successfully assigned input items.
.s
See the description of scanf() for further documentation.
.s
.i -8;Diagnostics
.s
Sscanf() returns -1 if the end of the input string was detected
and no data was stored.  It returns -2 if a palpably
incorrect format, such as "_%" is encountered.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strcat##String Concatenate
.pg
.hl 1 ^&String Concatenate\&
.s 2
.c ;**********
.c ;* strcat *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
strcat(out, in);
char		*out;
char		*in;
.s
.i -8;Description
.s
.fill
Append "in" to "out".  Return out.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strchr##Find First Instance of a Character in a String
.pg
.hl 1 ^&Find First Instance of a Character in a String\&
.s 2
.c ;**********
.c ;* strchr *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
strchr(stng, chr)
char		*stng;	/* String to search in	*/
char		chr;	/* Byte to search for	*/
.s
.i -8;Description
.s
.fill
If chr is in stng, return a pointer to it.  If not,
return NULL.
.s
See also index().
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strcmp##String Compare
.pg
.hl 1 ^&String Compare\&
.s 2
.c ;**********
.c ;* strcmp *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
strcmp(s1, s2);
char		*s1;
char		*s2;
.s
.i -8;Description
.s
.fill
Compare two strings, returning
.nf
	-1 if s1  < s2
	 0 if s1 == s2
	+1 if s1  > s2
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strcpy##String copy
.pg
.hl 1 ^&String copy\&
.s 2
.c ;**********
.c ;* strcpy *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
strcpy(out, in);
char		*out;
char		*in;
.s
.i -8;Description
.s
.fill
Copy "in" to "out".  Return out.
.s
.i -8;Bugs
.s
It might be more useful if it returned out + strlen(in).
(See cpystr()).
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########streq###String Equality Test
.pg
.hl 1 ^&String Equality Test\&
.s 2
.c ;*********
.c ;* streq *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
streq(a, b);
char		*a;
char		*b;
.s
.i -8;Description
.s
.fill
Return TRUE if the strings are equal.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strlen##String length
.pg
.hl 1 ^&String length\&
.s 2
.c ;**********
.c ;* strlen *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
strlen(s);
char		*s;
.s
.i -8;Description
.s
.fill
Return the length of the argument string.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strncat#String Concatenate
.pg
.hl 1 ^&String Concatenate\&
.s 2
.c ;***********
.c ;* strncat *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
strncat(out, in, count);
char		*out;
char		*in;
unsigned int	count;
.s
.i -8;Description
.s
.fill
Append "in" to "out".  Return out.
At most, "count" bytes are moved from in.
Note that "count" does not include the trailing null.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strncmp#String Compare With Count
.pg
.hl 1 ^&String Compare With Count\&
.s 2
.c ;***********
.c ;* strncmp *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
strncmp(s1, s2, count);
char		*s1;
char		*s2;
unsigned int	count;
.s
.i -8;Description
.s
.fill
Compare at most count bytes between two strings, returning:
.s
.nf
	-1 if s1  < s2
	 0 if s1 == s2
	+1 if s1  > s2
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strncpy#String Copy With Count
.pg
.hl 1 ^&String Copy With Count\&
.s 2
.c ;***********
.c ;* strncpy *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
strncpy(out, in, count);
char		*out;
char		*in;
unsigned int	count;
.s
.i -8;Description
.s
.fill
Copy "in" to "out".  Return out.
At most, "count" bytes are moved from in.
Note that "count" includes the trailing null.
.s
If strlen(in) is less than count, "out" is null-filled.
.s
If strlen(in) is greater than count, the output buffer
will not be null-trailed.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strneq##String Equality Test With Count
.pg
.hl 1 ^&String Equality Test With Count\&
.s 2
.c ;**********
.c ;* strneq *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
int
strneq(s1, s2, count);
char		*s1;
char		*s2;
unsigned int	count;
.s
.i -8;Description
.s
.fill
Compare at most count bytes between two strings, returning TRUE
if the the strings are equal.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########strrchr#Find Last Instance of a Character in a String
.pg
.hl 1 ^&Find Last Instance of a Character in a String\&
.s 2
.c ;***********
.c ;* strrchr *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
strrchr(stng, chr)
char		*stng;	/* String to search in	*/
char		chr;	/* Byte to search for	*/
.s
.i -8;Description
.s
.fill
If chr is in stng, return a pointer to the last (rightmost)
occurrance of it.  If not, return NULL.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########swabb###Byte swap (argument is a buffer pointer)
.pg
.hl 1 ^&Byte swap (argument is a buffer pointer)\&
.s 2
.c ;*********
.c ;* swabb *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
swabb(buffer);
char		*buffer;
.s
.i -8;Description:
.s
.fill
Return buffer[0] and buffer[1] as a byte-swapped integer.
Buffer need not be on any particular byte or word boundary.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########swabi###Byte swap, (argument is an integer)
.pg
.hl 1 ^&Byte swap, (argument is an integer)\&
.s 2
.c ;*********
.c ;* swabi *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
swabi(value);
int		value;
.s
.i -8;Description
.s
.fill
Return value byte-swab'ed.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########time####Compute time of day
.pg
.hl 1 ^&Compute time of day\&
.s 2
.c ;********
.c ;* time *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
.s
long
time(0);
.s
long
time(tloc);
long		*tloc;
.s
.i -8;Description
.s
.fill
time returns the time of day in seconds.  If tloc is
non-null, the return value is also stored in the place to
which tloc points.
.s
The value is the time since midnite, Jan 1, 1970, measured
in seconds.
.s
.i -8;Bugs
.s
The Unix time function returns GMT.  This function returns
local time.
.s
The leap year algorithm is only accurate in the range
.s
.nf
	1970 <= year < 2100
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########toascii#Convert to 7-bit Ascii
.pg
.hl 1 ^&Convert to 7-bit Ascii\&
.s 2
.c ;***********
.c ;* toascii *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
toascii(c);
int		c;
.s
.i -8;Description
.s
.fill
Remove the parity bit from c.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########tod#####Compute time of day
.pg
.hl 1 ^&Compute time of day\&
.s 2
.c ;*******
.c ;* tod *
.c ;*******
.s 2
.lm +8.nf
.i -8;Usage
.s
.s
long
tod(0);
.s
long
tod(tloc);
long		*tloc;
.s
.s
.i -8;Description
.s
.fill
tod() returns the time of day in seconds.  If tloc is
non-null, the return value is also stored in the place to
which tloc points.
.s
tod() is fairly fast.  time() computes the "Unix time" but
is much slower and larger.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########tolower#Convert upper-case to lower-case
.pg
.hl 1 ^&Convert upper-case to lower-case\&
.s 2
.c ;***********
.c ;* tolower *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
tolower(c);
int		c;
.s
.i -8;Description
.s
.fill
If c is a upper-case alphabetic, return it's lower-case
equivalent.  Otherwise, return c.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########toupper#Convert lower-case alphabetic to upper-case
.pg
.hl 1 ^&Convert lower-case alphabetic to upper-case\&
.s 2
.c ;***********
.c ;* toupper *
.c ;***********
.s 2
.lm +8.nf
.i -8;Usage
.s
toupper(c);
int		c;
.s
.i -8;Description
.s
.fill
If c is a lower-case alphabetic, return it's upper-case
equivalent.  Otherwise, return c.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########trace###Profile support entry module (with trace)
.pg
.hl 1 ^&Profile support entry module (with trace)\&
.s 2
.c ;*********
.c ;* trace *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
_#include <stdio.h>
extern	FILE	*$$flow;
.s
$$flow = fopen("trace.out", "w");
.s
/* or $$flow = stderr; to trace to the console	*/
_...
$$flow = NULL;		/* Turn off flow trace	*/
.s
.i -8;Description
.s
.fill
This module is called whenever a function that was compiled
with the profile option is executed.  It checks that the
stack will remain above 600 octal when the function executes
and optionally prints a flow-trace.
.s
If $$flow is set to a file descriptor, the function name,
location the function is called from, and the calling environment,
are printed on each call to a function compiled with the profile
option.
.s
The output format is:
.s
.nf
	function__name<TAB>caller<TAB>environment
.s
.fill
Where function__name is the name of the function being called,
caller is the address of the calling program (actually,
the return address), and environment is the R5 argument
pointer (for local addressing within the routine being called).
The information will be written to $$flow with a newline before
and after.
.s
If profiling has been enabled and the program exits via
error() or via an operating-system trap (such as an illegal
memory reference), a register dump and subroutine trace
will be written to stderr.
.s
See the description of calltrace() for details.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########traps###Operating system trap handlers
.pg
.hl 1 ^&Operating system trap handlers\&
.s 2
.c ;*********
.c ;* traps *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;####Internal
.s
$$sstt::		; RSX synchronous trap
			; vector
.s
$$t410::		; RT11 Entry after traps
			; to vectors 4 and 10
.s
.i -8;Description
.s
.fill
This module contains code which is executed after a synchronous
trap is detected by the operating system.  All such traps are
regarded as fatal errors and the program will be terminated.
.s
This code is enabled by executing any function, such as main(),
for which profiling was enabled when the function was compiled.
.s
The following traps are processed:
.s
.nf
	Illegal memory reference (trap through vector 4)
	Illegal instruction (trap through vector 10)
	BPT (assuming a debugging aid is not present)
	IOT
	Illegal TRAP (this may be a Fortran error)
	Illegal EMT (RSX only)
	Floating point exception (RSX only)
	Memory management violation
.s
.fill
Before exiting to the operating system, the registers at
the time the trap occurred will be written, in octal,
to stdout.  Note also that a calling sequence traceback will be
written to stdout by error().
.s
If a trap occurred, the program will exit to the operating
system with a "severe error" status.
.s
Note that some errors, such as stack overflow or random jumps
into the operating system, may cause the C program to be
terminated without the C library regaining control.
.s
.i -8;####Internal
.s
The first call to pcsv$ will initialze trapping.  On RSX11-M,
the SVTK$ executive service will be called.  On RT11, the
_.tprset monitor request will be executed.  As .trpset only
traps illegal memory reference (trap to 4) and illegal
instruction (trap to 10), the other interesting vectors
are initialized by absolute references (.ASECT).
.s
.i -8;Diagnostics
.s
.i -8;Bugs
.s
.i -8;####Internal
.s
Don't forget to handle floating-point when it's implemented.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ungetc##Push back a character onto an input file
.pg
.hl 1 ^&Push back a character onto an input file\&
.s 2
.c ;**********
.c ;* ungetc *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
ungetc(c, iop);
char		c;
FILE		*iop;
.s
.i -8;Description
.s
.fill
Push one character back on the indicated stream.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########unwind##Execute non-local goto
.pg
.hl 1 ^&Execute non-local goto\&
.s 2
.c ;**********
.c ;* unwind *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
setexit();
.s
unwind();
.s
.i -8;Description:
.s
.fill
These routines are useful for dealing with errors and interrupts
encountered in a low-level subroutine of a program.
.s
setexit() saves its stack environment in a static place for later
use by unwind().  It returns 0.
.s
Unwind() restores the environment saved by the last call of setexit().
It then returns in such a way that execution continues as if the
call of setexit() had just returned.  All accessible data have values
as of the time unwind() was called.  The return from setexit() will
have the value 1.  For example:
.s
.nf
	if (setexit()) {
			/* Unwind called	*/
	}
	else {
			/* Setexit setup called */
	}
.s
.fill
The routine that called setexit() must still be active when unwind()
is called.
.s
.i -8;Bugs
.s
Obsolete -- use setjmp/longjmp instead.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########wdleng##Expensive way to say sizeof(int)
.pg
.hl 1 ^&Expensive way to say sizeof(int)\&
.s 2
.c ;**********
.c ;* wdleng *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
wdleng()
.s
.i -8;Description
.s
.fill
Return word length in bits.
.s
.i -8;Bugs
.s
Assuming 8-bit bytes, this may be replaced by:
.s
.nf
	_#define wdleng() (sizeof (int) * 8)
.s
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########wrapup##Dummy routine called on exit
.pg
.hl 1 ^&Dummy routine called on exit\&
.s 2
.c ;**********
.c ;* wrapup *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
wrapup()
.s
.i -8;Description
.s
.fill
This routine (which does nothing) is called on exit if the user's
program does not provide a wrapup() routine.
.s
Wrapup will be called only once.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########zero####Clear a block of memory
.pg
.hl 1 ^&Clear a block of memory\&
.s 2
.c ;********
.c ;* zero *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
zero(addr, nbytes);
char		*addr;
unsigned int	nbytes;
.s
.i -8;Description:
.s
.fill
Clear the block of core.  No value is returned.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy

